@@ -151,6 +151,8 @@ void *idrecDataInit(int t)
 #endif
     case BIGINTMAT_CMD:
       return (void *)new bigintmat();
+    case BIGINTVEC_CMD:
+      return (void *)new bigintmat(1,0,coeffs_BIGINT);
     case BUCKET_CMD:
       if (currRing!=NULL)
       return (void*)sBucketCreate(currRing);
@@ -908,8 +910,16 @@ BOOLEAN iiAlias(leftv p)
          break;
        // case ring: cannot happen
        default:
-         Werror("unknown type %d",p->Typ());
-         return TRUE;
+         {
+           int t=p->Typ();
+           if (t<MAX_TOK)
+           {
+             Werror("unknown type %d(%s)",t,Tok2Cmdname(t));
+             return TRUE;
+           }
+           /*else: blackbox type, not yet set*/
+           break;
+         }
   }
   pp->typ=ALIAS_CMD;
   IDDATA(pp)=(char*)h->data;
