@@ -14,6 +14,7 @@
 
 #include "reporter/si_signals.h"
 #include "reporter/s_buff.h"
+#include "reporter/si_signals.h"
 
 #include "coeffs/bigintmat.h"
 #include "coeffs/longrat.h"
@@ -24,6 +25,7 @@
 #include "polys/simpleideals.h"
 #include "polys/matpol.h"
 
+#include "kernel/oswrapper/timer.h"
 #include "kernel/oswrapper/timer.h"
 #include "kernel/oswrapper/feread.h"
 #include "kernel/oswrapper/rlimit.h"
@@ -49,7 +51,8 @@
 #include <netdb.h>
 #include <netinet/in.h> /* for htons etc.*/
 
-#define SSI_VERSION 14
+
+#define SSI_VERSION 15
 // 5->6: changed newstruct representation
 // 6->7: attributes
 // 7->8: qring
@@ -59,12 +62,13 @@
 // 11->12: add rank to ideal/module, add smatrix
 // 12->13: NC rings
 // 13->14: ring references
+// 14->15: bigintvec, prune_map, mres_map
 
+EXTERN_VAR BOOLEAN FE_OPT_NO_SHELL_FLAG;
 VAR link_list ssiToBeClosed=NULL;
 VAR volatile BOOLEAN ssiToBeClosed_inactive=TRUE;
 
 // forward declarations:
-static void ssiWriteIdeal(const ssiInfo *d, int typ,const ideal I);
 static void ssiWritePoly_R(const ssiInfo *d, int typ, poly p, const ring r);
 static void ssiWriteIdeal_R(const ssiInfo *d, int typ,const ideal I, const ring r);
 static poly ssiReadPoly_R(const ssiInfo *D, const ring r);
@@ -147,7 +151,7 @@ static void ssiCheckCurrRing(const ring r)
   assume((currRing==r) || rEqual(r,currRing));
 }
 // the implementation of the functions:
-static void ssiWriteInt(const ssiInfo *d,const int i)
+void ssiWriteInt(const ssiInfo *d,const int i)
 {
   fprintf(d->f_write,"%d ",i);
   //if (d->f_debug!=NULL) fprintf(d->f_debug,"int: %d ",i);
@@ -405,7 +409,7 @@ static void ssiWriteIdeal_R(const ssiInfo *d, int typ,const ideal I, const ring
      ssiWritePoly_R(d,tt,I->m[i],R);
    }
 }
-static void ssiWriteIdeal(const ssiInfo *d, int typ,const ideal I)
+void ssiWriteIdeal(const ssiInfo *d, int typ,const ideal I)
 {
   ssiWriteIdeal_R(d,typ,I,d->r);
 }
@@ -473,6 +477,16 @@ static void ssiWriteBigintmat(const ssiInfo *d,bigintmat * v)
   }
 }
 
+static void ssiWriteBigintvec(const ssiInfo *d,bigintmat * v)
+{
+  fprintf(d->f_write,"%d ",v->cols());
+  int i;
+  for(i=0;i<v->length();i++)
+  {
+    ssiWriteBigInt(d,(*v)[i]);
+  }
+}
+
 static char *ssiReadString(const ssiInfo *d)
 {
   char *buf;
@@ -486,9 +500,9 @@ static char *ssiReadString(const ssiInfo *d)
   return buf;
 }
 
-static int ssiReadInt(s_buff fich)
+int ssiReadInt(const ssiInfo *d)
 {
-  return s_readint(fich);
+  return s_readint(d->f_read);
 }
 
 static number ssiReadNumber_CF(const ssiInfo *d, const coeffs cf)
@@ -704,7 +718,7 @@ static poly ssiReadPoly_R(const ssiInfo *d, const ring r)
 {
 // < # of terms> < term1> < .....
   int n,i,l;
-  n=ssiReadInt(d->f_read); // # of terms
+  n=ssiReadInt(d); // # of terms
   //Print("poly: terms:%d\n",n);
   poly p;
   poly ret=NULL;
@@ -750,7 +764,7 @@ static ideal ssiReadIdeal_R(const ssiInfo *d,const ring r)
   return I;
 }
 
-static ideal ssiReadIdeal(ssiInfo *d)
+ideal ssiReadIdeal(ssiInfo *d)
 {
   return ssiReadIdeal_R(d,d->r);
 }
@@ -880,12 +894,24 @@ static bigintmat* ssiReadBigintmat(const ssiInfo *d)
   }
   return v;
 }
+static bigintmat* ssiReadBigintvec(const ssiInfo *d)
+{
+  int c;
+  c=s_readint(d->f_read);
+  bigintmat *v=new bigintmat(1,c,coeffs_BIGINT);
+  for(int i=0;i<c;i++)
+  {
+    (*v)[i]=ssiReadBigInt(d);
+  }
+  return v;
+}
 
 static void ssiReadBlackbox(leftv res, si_link l)
 {
   ssiInfo *d=(ssiInfo*)l->data;
-  int throwaway=s_readint(d->f_read);
-  char *name=ssiReadString(d);
+  leftv lv=ssiRead1(l);
+  char *name=(char*)lv->data;
+  omFreeBin(lv,sleftv_bin);
   int tok;
   blackboxIsCmd(name,tok);
   if (tok>MAX_TOK)
@@ -1001,16 +1027,21 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
     {
       if (strcmp(mode,"fork")==0)
       {
+        int pc[2];
+        int cp[2];
+        int err1=pipe(pc);
+        int err2=pipe(cp);
+        if (err1 || err2)
+        {
+          Werror("pipe failed with %d\n",errno);
+          return TRUE;
+        }
         link_list n=(link_list)omAlloc(sizeof(link_struct));
         n->u=u;
         n->l=l;
         n->next=(void *)ssiToBeClosed;
         ssiToBeClosed=n;
 
-        int pc[2];
-        int cp[2];
-        pipe(pc);
-        pipe(cp);
         pid_t pid = fork();
         if (pid == -1 && errno == EAGAIN)   // RLIMIT_NPROC too low?
         {
@@ -1028,6 +1059,7 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
           sigemptyset(&sigint);
           sigaddset(&sigint, SIGINT);
           sigprocmask(SIG_BLOCK, &sigint, NULL);
+          si_set_signal(SIGTERM,sig_term_hdl);
           /* set #cpu to 1 for the child:*/
           feSetOptValue(FE_OPT_CPUS,1);
 
@@ -1169,7 +1201,7 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
       }
     }
     // =========================================================================
-    else /*l->name=NULL*/
+    else /*now l->name!=NULL*/
     {
       // tcp mode
       if(strcmp(mode,"tcp")==0)
@@ -1225,7 +1257,10 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
         }
         char* ssh_command = (char*)omAlloc(256);
         char* ser_host = (char*)omAlloc(64);
-        gethostname(ser_host,64);
+        if(strcmp(cli_host,"localhost")==0)
+          strcpy(ser_host,"localhost");
+        else
+          gethostname(ser_host,64);
         if (strcmp(cli_host,"localhost")==0) /*avoid "ssh localhost" as key may change*/
           snprintf(ssh_command,256,"%s -q --batch --link=ssi --MPhost=%s --MPport=%d &",path,ser_host,portno);
         else
@@ -1234,7 +1269,15 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
         omFree(path);
         omFree(cli_host);
         if (TEST_OPT_PROT) { Print("running >>%s<<\n",ssh_command); }
-        system(ssh_command);
+        int re=system(ssh_command);
+        if (re<0)
+        {
+          Werror("ERROR running `%s` (%d)",ssh_command,re);
+          l->data=NULL;
+          l->flags=0;
+          omFree(d);
+          return TRUE;
+        }
         omFree(ssh_command);
         omFree(ser_host);
         clilen = sizeof(cli_addr);
@@ -1320,6 +1363,7 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
       else
       {
         // normal link to a file
+        if (FE_OPT_NO_SHELL_FLAG) {WerrorS("no links allowed");return TRUE;}
         FILE *outfile;
         char *filename=l->name;
 
@@ -1398,6 +1442,7 @@ BOOLEAN ssiClose(si_link l)
       {
         fputs("99\n",d->f_write);
         fflush(d->f_write);
+        d->quit_sent=1;
       }
       // clean ring
       if (d->r!=NULL) rKill(d->r);
@@ -1406,50 +1451,39 @@ BOOLEAN ssiClose(si_link l)
         if (d->rings[i]!=NULL)  rKill(d->rings[i]);
         d->rings[i]=NULL;
       }
-      // did the child to stop ?
-      si_waitpid(d->pid,NULL,WNOHANG);
-      if ((d->pid!=0)
-      && (kill(d->pid,0)==0)) // child is still running
+      if (d->f_read!=NULL) { s_close(d->f_read);d->f_read=NULL;}
+      if (d->f_write!=NULL) { fclose(d->f_write); d->f_write=NULL; }
+      if (((strcmp(l->mode,"tcp")==0)
+      || (strcmp(l->mode,"fork")==0))
+      && (d->pid>1))
       {
-        struct timespec t;
-        t.tv_sec=0;
-        t.tv_nsec=100000000; // <=100 ms
-        struct timespec rem;
-        int r;
-        loop
+        // did the child stop ?
+        int pid=si_waitpid(d->pid,NULL,WNOHANG);
+        if ((pid==0) /* no status change for child*/
+        && (kill(d->pid,0)==0)) // child is still running
         {
-          // wait till signal or time rem:
-          r = nanosleep(&t, &rem);
-          t = rem;
-          // child finished:
-          if (si_waitpid(d->pid,NULL,WNOHANG) != 0) break;
-          // other signal, waited s>= 100 ms:
-          if ((r==0) || (errno != EINTR)) break;
-        }
-        if (kill(d->pid,0) == 0) // pid still exists
-        {
-          kill(d->pid,15);
-          t.tv_sec=5; // <=5s
-          t.tv_nsec=0;
-          loop
+          struct timespec t;
+          struct timespec rem;
+          // wait 1 sec
+          for(int i=0;i<10;i++)
           {
-            // wait till signal or time rem:
-            r = nanosleep(&t, &rem);
-            t = rem;
-            // child finished:
-            if (si_waitpid(d->pid,NULL,WNOHANG) != 0) break;
-            // other signal, waited s>=  5 s:
-            if ((r==0) || (errno != EINTR)) break;
+            // wait till signal or 100ms:
+            t.tv_sec=0;
+            t.tv_nsec=100000000; // <=100 ms
+            nanosleep(&t, &rem);
+            // child finished ?
+            if (si_waitpid(d->pid,NULL,WNOHANG) == d->pid) break;
           }
-          if (kill(d->pid,0) == 0)
+          if (kill(d->pid,0)==0) // child still exists
           {
-            kill(d->pid,9); // just to be sure
-            si_waitpid(d->pid,NULL,0);
+            kill(d->pid,SIGTERM);
+            t.tv_sec=1;
+            t.tv_nsec=0; // <=1000 ms
+            nanosleep(&t, &rem);
+            si_waitpid(d->pid,NULL,WNOHANG);
           }
         }
       }
-      if (d->f_read!=NULL) { s_close(d->f_read);d->f_read=NULL;}
-      if (d->f_write!=NULL) { fclose(d->f_write); d->f_write=NULL; }
       if ((strcmp(l->mode,"tcp")==0)
       || (strcmp(l->mode,"fork")==0))
       {
@@ -1493,21 +1527,26 @@ leftv ssiRead1(si_link l)
   switch(t)
   {
     case 1:res->rtyp=INT_CMD;
-           res->data=(char *)(long)ssiReadInt(d->f_read);
+           res->data=(char *)(long)ssiReadInt(d);
+           //Print("int: %d\n",(int)(long)res->data);
            break;
     case 2:res->rtyp=STRING_CMD;
            res->data=(char *)ssiReadString(d);
+           //Print("str: %s\n",(char*)res->data);
            break;
     case 3:res->rtyp=NUMBER_CMD;
            if (d->r==NULL) goto no_ring;
            ssiCheckCurrRing(d->r);
            res->data=(char *)ssiReadNumber(d);
+           //Print("number\n");
            break;
     case 4:res->rtyp=BIGINT_CMD;
            res->data=(char *)ssiReadBigInt(d);
+           //Print("bigint\n");
            break;
     case 15:
     case 5:{
+           //Print("ring %d\n",t);
              d->r=ssiReadRing(d);
              if (errorreported) return NULL;
              res->data=(char*)d->r;
@@ -1522,21 +1561,25 @@ leftv ssiRead1(si_link l)
            }
            break;
     case 6:res->rtyp=POLY_CMD;
+           //Print("poly\n");
            if (d->r==NULL) goto no_ring;
            ssiCheckCurrRing(d->r);
            res->data=(char*)ssiReadPoly(d);
            break;
     case 7:res->rtyp=IDEAL_CMD;
+           //Print("ideal\n");
            if (d->r==NULL) goto no_ring;
            ssiCheckCurrRing(d->r);
            res->data=(char*)ssiReadIdeal(d);
            break;
     case 8:res->rtyp=MATRIX_CMD;
+           //Print("matrix\n");
            if (d->r==NULL) goto no_ring;
            ssiCheckCurrRing(d->r);
            res->data=(char*)ssiReadMatrix(d);
            break;
     case 9:res->rtyp=VECTOR_CMD;
+           //Print("vector\n");
            if (d->r==NULL) goto no_ring;
            ssiCheckCurrRing(d->r);
            res->data=(char*)ssiReadPoly(d);
@@ -1544,6 +1587,7 @@ leftv ssiRead1(si_link l)
     case 10:
     case 22:if (t==22) res->rtyp=SMATRIX_CMD;
            else        res->rtyp=MODUL_CMD;
+           //Print("module/smatrix %d\n",t);
            if (d->r==NULL) goto no_ring;
            ssiCheckCurrRing(d->r);
            {
@@ -1555,6 +1599,7 @@ leftv ssiRead1(si_link l)
            break;
     case 11:
            {
+           //Print("cmd\n",t);
              res->rtyp=COMMAND;
              res->data=ssiReadCommand(l);
              int nok=res->Eval();
@@ -1563,6 +1608,7 @@ leftv ssiRead1(si_link l)
            }
     case 12: /*DEF_CMD*/
            {
+           //Print("def\n",t);
              res->rtyp=0;
              res->name=(char *)ssiReadString(d);
              int nok=res->Eval();
@@ -1593,6 +1639,9 @@ leftv ssiRead1(si_link l)
     case 23: ssiReadRingProperties(l);
              return ssiRead1(l);
              break;
+    case 24: res->rtyp=BIGINTVEC_CMD;
+             res->data=ssiReadBigintvec(d);
+             break;
     // ------------
     case 98: // version
              {
@@ -1616,7 +1665,8 @@ leftv ssiRead1(si_link l)
                 omFreeBin(res,sleftv_bin);
                 return ssiRead1(l);
              }
-    case 99: omFreeBin(res,sleftv_bin); ssiClose(l); m2_end(0);
+    case 99: omFreeBin(res,sleftv_bin); ssiClose(l); m2_end(-1);
+             break; /*to make compiler happy*/
     case 0: if (s_iseof(d->f_read))
             {
               ssiClose(l);
@@ -1751,11 +1801,13 @@ BOOLEAN ssiWrite(si_link l, leftv data)
                         }
                         if(tt==IDEAL_CMD)       fputs("7 ",d->f_write);
                         else if(tt==MATRIX_CMD) fputs("8 ",d->f_write);
-                        else if(tt==SMATRIX_CMD) fputs("22 ",d->f_write);
-                        else /* tt==MODUL_CMD*/
+                        else /* tt==MODUL_CMD, SMATRIX_CMD*/
                         {
                           ideal M=(ideal)dd;
-                          fprintf(d->f_write,"10 %d ",(int)M->rank);
+                          if (tt==MODUL_CMD)
+                            fprintf(d->f_write,"10 %d ",(int)M->rank);
+                          else /*(tt==SMATRIX_CMD)*/
+                            fprintf(d->f_write,"22 %d ",(int)M->rank);
                         }
                         ssiWriteIdeal(d,tt,(ideal)dd);
                         break;
@@ -1787,6 +1839,10 @@ BOOLEAN ssiWrite(si_link l, leftv data)
                    fputs("19 ",d->f_write);
                    ssiWriteBigintmat(d,(bigintmat *)dd);
                    break;
+          case BIGINTVEC_CMD:
+                   fputs("24 ",d->f_write);
+                   ssiWriteBigintvec(d,(bigintmat *)dd);
+                   break;
           default:
             if (tt>MAX_TOK)
             {
@@ -1838,9 +1894,31 @@ const char* slStatusSsi(si_link l, const char* request)
   ||(strcmp(l->mode,"connect")==0))
   && (strcmp(request, "read") == 0))
   {
+    if (s_isready(d->f_read)) return "ready";
+#if defined(HAVE_POLL)
+    pollfd pfd;
+    loop
+    {
+      /* Don't block. Return socket status immediately. */
+      pfd.fd=d->fd_read;
+      pfd.events=POLLIN;
+      //Print("test fd %d\n",d->fd_read);
+      /* check with select: chars waiting: no -> not ready */
+      switch (si_poll(&pfd,1,0))
+      {
+        case 0: /* not ready */ return "not ready";
+        case -1: /*error*/      return "error";
+        case 1: /*ready ? */    break;
+      }
+#else
     fd_set  mask;
     struct timeval wt;
-    if (s_isready(d->f_read)) return "ready";
+    if (FD_SETSIZE<=d->fd_read)
+    {
+      Werror("file descriptor number too high (%d)",d->fd_read);
+      return "error";
+    }
+
     loop
     {
       /* Don't block. Return socket status immediately. */
@@ -1850,15 +1928,16 @@ const char* slStatusSsi(si_link l, const char* request)
       FD_ZERO(&mask);
       FD_SET(d->fd_read, &mask);
       //Print("test fd %d\n",d->fd_read);
-    /* check with select: chars waiting: no -> not ready */
+      /* check with select: chars waiting: no -> not ready */
       switch (si_select(d->fd_read+1, &mask, NULL, NULL, &wt))
       {
         case 0: /* not ready */ return "not ready";
         case -1: /*error*/      return "error";
         case 1: /*ready ? */    break;
       }
-    /* yes: read 1 char*/
-    /* if \n, check again with select else ungetc(c), ready*/
+#endif
+      /* yes: read 1 char*/
+      /* if \n, check again with select else ungetc(c), ready*/
       int c=s_getc(d->f_read);
       //Print("try c=%d\n",c);
       if (c== -1) return "eof"; /* eof or error */
@@ -1885,25 +1964,107 @@ const char* slStatusSsi(si_link l, const char* request)
   else return "unknown status request";
 }
 
-int slStatusSsiL(lists L, int timeout)
+int slStatusSsiL(lists L, int timeout, BOOLEAN *ignore)
 {
 // input: L: a list with links of type
 //           ssi-connect, ssi-fork, ssi-tcp, MPtcp-fork or MPtcp-launch.
 //           Note: Not every entry in L must be set.
-//        timeout: timeout for select in micro-seconds
+//        timeout: timeout for select in milli-seconds
 //           or -1 for infinity
 //           or 0 for polling
 // returns: ERROR (via Werror): L has wrong elements or link not open
-//           -2: select returns an error
+//           -2: error in L
 //           -1: the read state of all links is eof
 //           0:  timeout (or polling): none ready,
 //           i>0: (at least) L[i] is ready
+#if defined(HAVE_POLL) && !defined(__APPLE__)
+// fd is restricted on OsX by ulimit "file descriptors" (256)
   si_link l;
-  ssiInfo *d;
+  ssiInfo *d=NULL;
   int d_fd;
-  fd_set  mask, fdmask;
+  int s;
+  int nfd=L->nr+1;
+  pollfd *pfd=(pollfd*)omAlloc0(nfd*sizeof(pollfd));
+  for(int i=L->nr; i>=0; i--)
+  {
+    pfd[i].fd=-1;
+    if (L->m[i].Typ()!=DEF_CMD)
+    {
+      if (L->m[i].Typ()!=LINK_CMD)
+      { WerrorS("all elements must be of type link"); return -2;}
+      l=(si_link)L->m[i].Data();
+      if(SI_LINK_OPEN_P(l)==0)
+      { WerrorS("all links must be open"); return -2;}
+      if (((strcmp(l->m->type,"ssi")!=0) && (strcmp(l->m->type,"MPtcp")!=0))
+      || ((strcmp(l->mode,"fork")!=0) && (strcmp(l->mode,"tcp")!=0)
+        && (strcmp(l->mode,"launch")!=0) && (strcmp(l->mode,"connect")!=0)))
+      {
+        WerrorS("all links must be of type ssi:fork, ssi:tcp, ssi:connect");
+        return -2;
+      }
+      if (strcmp(l->m->type,"ssi")==0)
+      {
+        d=(ssiInfo*)l->data;
+        d_fd=d->fd_read;
+        if (!s_isready(d->f_read))
+        {
+          pfd[i].fd=d_fd;
+          pfd[i].events=POLLIN;
+        }
+        else
+        {
+          return i+1;
+        }
+      }
+      else
+      {
+        Werror("wrong link type >>%s<<",l->m->type);
+        return -2;
+      }
+    }
+    else if (ignore!=NULL)
+    {
+      ignore[i]=TRUE; // not a link
+    }
+  }
+  s=si_poll(pfd,nfd,timeout);
+  if (s==-1)
+  {
+    Werror("error in poll call (errno:%d)",errno);
+    return -2; /*error*/
+  }
+  if(s==0)
+  {
+    return 0; /*timeout*/
+  }
+  for(int i=L->nr; i>=0; i--)
+  {
+    if ((L->m[i].rtyp==LINK_CMD)
+    && ((ignore==NULL)||(ignore[i]==FALSE)))
+    {
+      // the link type is ssi, that's already tested
+      l=(si_link)L->m[i].Data();
+      d=(ssiInfo*)l->data;
+      d_fd=d->fd_read;
+      if (pfd[i].fd==d_fd)
+      {
+        if (pfd[i].revents &POLLIN)
+        {
+          omFree(pfd);
+          return i+1;
+        }
+      }
+    }
+  }
+  // no ready
+  return 0;
+#else
+  // fd is restricted to <=1024
+  si_link l;
+  ssiInfo *d=NULL;
+  int d_fd;
+  fd_set fdmask;
   FD_ZERO(&fdmask);
-  FD_ZERO(&mask);
   int max_fd=0; /* 1 + max fd in fd_set */
 
   /* timeout */
@@ -1916,8 +2077,8 @@ int slStatusSsiL(lists L, int timeout)
   }
   else
   {
-    wt.tv_sec  = timeout / 1000000;
-    wt.tv_usec = timeout % 1000000;
+    wt.tv_sec  = timeout / 1000;
+    wt.tv_usec = (timeout % 1000)*1000;
   }
 
   /* auxiliary variables */
@@ -1951,8 +2112,11 @@ int slStatusSsiL(lists L, int timeout)
         d_fd=d->fd_read;
         if (!s_isready(d->f_read))
         {
-          FD_SET(d_fd, &fdmask);
-          if (d_fd > max_fd) max_fd=d_fd;
+          if ((ignore==NULL) || (ignore[i]==FALSE))
+          {
+            FD_SET(d_fd, &fdmask);
+            if (d_fd > max_fd) max_fd=d_fd;
+          }
         }
         else
           return i+1;
@@ -1965,23 +2129,17 @@ int slStatusSsiL(lists L, int timeout)
     }
   }
   max_fd++;
-
-do_select:
-  /* copy fdmask to mask */
-  FD_ZERO(&mask);
-  for(k = 0; k < max_fd; k++)
+  if (FD_SETSIZE<=max_fd)
   {
-    if(FD_ISSET(k, &fdmask))
-    {
-      FD_SET(k, &mask);
-    }
+    Werror("file descriptor number too high (%d)",max_fd);
+    return -2;
   }
 
   /* check with select: chars waiting: no -> not ready */
-  s = si_select(max_fd, &mask, NULL, NULL, wt_ptr);
+  s = si_select(max_fd, &fdmask, NULL, NULL, wt_ptr);
   if (s==-1)
   {
-    WerrorS("error in select call");
+    Werror("error in select call (errno:%d)",errno);
     return -2; /*error*/
   }
   if (s==0)
@@ -1991,7 +2149,7 @@ do_select:
   else /* s>0, at least one ready  (the number of fd which are ready is s)*/
   {
     j=0;
-    while (j<=max_fd) { if (FD_ISSET(j,&mask)) break; j++; }
+    while (j<=max_fd) { if (FD_ISSET(j,&fdmask)) break; j++; }
     for(i=L->nr; i>=0; i--)
     {
       if (L->m[i].rtyp==LINK_CMD)
@@ -2001,60 +2159,13 @@ do_select:
         {
           d=(ssiInfo*)l->data;
           d_fd=d->fd_read;
-          if(j==d_fd) break;
-        }
-        else
-        {
-          Werror("wrong link type >>%s<<",l->m->type);
-          return -2;
+          if(j==d_fd) return i+1;
         }
       }
     }
-    // only ssi links:
-    loop
-    {
-      /* yes: read 1 char*/
-      /* if \n, check again with select else ungetc(c), ready*/
-      /* setting: d: current ssiInfo, j current fd, i current entry in L*/
-      int c=s_getc(d->f_read);
-      //Print("try c=%d\n",c);
-      if (c== -1) /* eof */
-      {
-        FD_CLR(j,&fdmask);
-        fdmaskempty = 1;
-        for(k = 0; k < max_fd; k++)
-        {
-          if(FD_ISSET(k, &fdmask))
-          {
-            fdmaskempty = 0;
-            break;
-          }
-        }
-        if(fdmaskempty)
-        {
-          return -1;
-        }
-        if(timeout != -1)
-        {
-          timeout = si_max(0,
-             timeout - 1000000*(getRTimer()/TIMER_RESOLUTION - startingtime));
-          wt.tv_sec  = timeout / 1000000;
-          wt.tv_usec = (timeout % 1000000);
-        }
-        goto do_select;
-      }
-
-      else if (isdigit(c))
-      { s_ungetc(c,d->f_read); return i+1; }
-      else if (c>' ')
-      {
-        Werror("unknown char in ssiLink(%d)",c);
-        return -2;
-      }
-      /* else: next char */
-      goto do_select;
-    }
   }
+  return 0;
+#endif
 }
 
 int ssiBatch(const char *host, const char * port)
@@ -2085,7 +2196,7 @@ int ssiBatch(const char *host, const char * port)
     omFreeBin(h, sleftv_bin);
   }
   /* never reached*/
-  exit(0);
+  _exit(0);
 }
 
 STATIC_VAR int ssiReserved_P=0;
@@ -2204,6 +2315,7 @@ void sig_chld_hdl(int)
   pid_t kidpid;
   int status;
 
+#if 0
   loop
   {
     kidpid = si_waitpid(-1, &status, WNOHANG);
@@ -2239,6 +2351,7 @@ void sig_chld_hdl(int)
       else hh=(link_list)hh->next;
     }
   }
+#endif
 }
 
 static BOOLEAN DumpSsiIdhdl(si_link l, idhdl h)
@@ -2376,6 +2489,26 @@ BOOLEAN ssiGetDump(si_link l)
   }
   return FALSE;
 }
+
+void singular_close_links()
+{
+  link_list hh=ssiToBeClosed;
+  while(hh!=NULL)
+  {
+    if ((hh->l->m!=NULL)
+    && (hh->l->m->Open==ssiOpen)
+    && SI_LINK_OPEN_P(hh->l)
+    && (strcmp(hh->l->mode, "fork")==0))
+    {
+      SI_LINK_SET_CLOSE_P(hh->l);
+      ssiInfo *d = (ssiInfo *)hh->l->data;
+      if (d->f_read!=NULL) { s_close(d->f_read);d->f_read=NULL;}
+      if (d->f_write!=NULL) { fclose(d->f_write); d->f_write=NULL; }
+    }
+    hh=(link_list)hh->next;
+  }
+  ssiToBeClosed=NULL;
+}
 // ----------------------------------------------------------------
 // format
 // 1 int %d
@@ -2397,12 +2530,13 @@ BOOLEAN ssiGetDump(si_link l)
 // 17 intvec <len> ...
 // 18 intmat
 // 19 bigintmat <r> <c> ...
-// 20 blackbox <name> 1 <len> ...
+// 20 blackbox <name> <len> ...
 // 21 attrib <bit-attrib> <len> <a-name1> <val1>... <data>
 // 22 smatrix
 // 23 0 <log(bitmask)> ring properties: max.exp.
 // 23 1 <log(bitmask)> <r->IsLPRing> ring properties:LPRing
 // 23 2 <matrix C> <matrix D> ring properties: PLuralRing
+// 24 bigintvec <c>
 //
 // 98: verify version: <ssi-version> <MAX_TOK> <OPT1> <OPT2>
 // 99: quit Singular
