@@ -14,6 +14,7 @@
 #define PLURAL_INTERNAL_DECLARATIONS 1
 
 #include "kernel/mod2.h"
+#include "kernel/oswrapper/vspace.h"
 #include "misc/sirandom.h"
 #include "omalloc/omalloc.h"
 #include "misc/mylimits.h"
@@ -466,7 +467,7 @@ void singular_example(char *str)
     char *res_m=feResource('m', 0);
     if (res_m!=NULL)
     {
-      sprintf(sing_file, "%s/%s.sing", res_m, s);
+      snprintf(sing_file,MAXPATHLEN, "%s/%s.sing", res_m, s);
       fd = feFopen(sing_file, "r");
     }
     if (fd != NULL)
@@ -561,6 +562,7 @@ const struct soptionStruct verboseStruct[]=
   {"warn",     Sy_bit(V_ALLWARN),   ~Sy_bit(V_ALLWARN)},
   {"intersectSyz",Sy_bit(V_INTERSECT_SYZ), ~Sy_bit(V_INTERSECT_SYZ)},
   {"intersectElim",Sy_bit(V_INTERSECT_ELIM), ~Sy_bit(V_INTERSECT_ELIM)},
+  {"no_try_HC",Sy_bit(V_NO_TRY_HC), ~Sy_bit(V_NO_TRY_HC)},
 /*special for "none" and also end marker for showOption:*/
   {"ne",         0,          0 }
 };
@@ -869,6 +871,9 @@ char * versionString(/*const bool bShowDetails = false*/ )
 #endif
 #ifdef KDEBUG
               StringAppendS("KDEBUG,");
+#endif
+#ifdef HAVE_SDB
+              StringAppendS("sdb,");
 #endif
               StringAppendS("\n\t");
 #ifdef __OPTIMIZE__
@@ -1110,7 +1115,7 @@ void m2_end(int i)
       {
         int pid=getpid();
         char buf[20];
-        sprintf(buf,"/tmp/sing_log.%d",pid);
+        snprintf(buf,20,"/tmp/sing_log.%d",pid);
         remove(buf);
       }
     }
@@ -1426,7 +1431,7 @@ void siInit(char *name)
 #endif
 
 // random generator: -----------------------------------------------
-  int t=initTimer();
+  int t=startTimer();
   if (t==0) t=1;
   initRTimer();
   siSeed=t;
@@ -1450,6 +1455,9 @@ void siInit(char *name)
   #elif defined(_SC_NPROCESSORS_CONF)
   if ((cpu_n=sysconf(_SC_NPROCESSORS_CONF))>cpus) cpus=cpu_n;
   #endif
+  #ifdef HAVE_VSPACE
+  if (cpus>vspace::internals::MAX_PROCESS) cpus=vspace::internals::MAX_PROCESS;
+  #endif
   feSetOptValue(FE_OPT_CPUS, cpus);
 // how many threads ? -----------------------------------------------------
   feSetOptValue(FE_OPT_THREADS, cpus);
