@@ -14,6 +14,7 @@
 #include "kernel/oswrapper/feread.h"
 #include "Singular/fevoices.h"
 #include "kernel/oswrapper/timer.h"
+#include "kernel/oswrapper/vspace.h"
 
 #include "ipshell.h"
 #include "cntrlc.h"
@@ -97,6 +98,12 @@ int main(          /* main entry to Singular */
       errormsg = feSetOptValue((feOptIndex) option_index, (int) 1);
     else
       errormsg = feSetOptValue((feOptIndex) option_index, fe_optarg);
+    // not more than MAX_PROCESS cpus
+    long cpus=(long)feOptValue(FE_OPT_CPUS);
+    #ifdef HAVE_VSPACE
+    if (cpus>vspace::internals::MAX_PROCESS) cpus=vspace::internals::MAX_PROCESS;
+    #endif
+    feSetOptValue(FE_OPT_CPUS,cpus);
 
     if (errormsg)
     {
@@ -137,7 +144,7 @@ int main(          /* main entry to Singular */
     if (feOptValue(FE_OPT_NO_SHELL))
     {
       WarnS("running in restricted mode:"
-      " shell invocation and links are disallowed");
+      " shell invocation and file links are disallowed");
       FE_OPT_NO_SHELL_FLAG=TRUE;
     }
   }
@@ -154,7 +161,8 @@ int main(          /* main entry to Singular */
   {
     int pid=getpid();
     char *buf=(char*)malloc(10+strlen((char*) feOptSpec[FE_OPT_LOG].value));
-    sprintf(buf,"%s.%d",(char*) feOptSpec[FE_OPT_LOG].value,pid);
+    snprintf(buf,10+strlen((char*) feOptSpec[FE_OPT_LOG].value),
+      "%s.%d",(char*) feOptSpec[FE_OPT_LOG].value,pid);
     File_Log=fopen(buf,"w");
   }
 
