@@ -12,10 +12,13 @@
 #include <string.h>
 
 #ifdef HAVE_VSPACE
+#include "reporter/si_signals.h"
 #include "kernel/oswrapper/vspace.h"
+#include "Singular/cntrlc.h"
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
+#include "Singular/links/ssiLink.h"
 #endif
 
 BOOLEAN kVerify1(ideal F, ideal Q)
@@ -212,6 +215,8 @@ BOOLEAN kVerify2(ideal F, ideal Q)
   if (cpus>=vspace::internals::MAX_PROCESS)
     cpus=vspace::internals::MAX_PROCESS-1;
   /* start no more than MAX_PROCESS-1 children */
+  if (cpus>strat->Ll) cpus=strat->Ll;
+  /* start no more children than elements in L */
   int parent_pid=getpid();
   using namespace vspace;
   vmem_init();
@@ -222,7 +227,7 @@ BOOLEAN kVerify2(ideal F, ideal Q)
   {
    queue->enqueue(i); // the tasks: process pair L[i]
   }
-  for(int i=cpus;i>=0;i--)
+  for(int i=cpus*2;i>=0;i--)
   {
     queue->enqueue(-1); // stop sign, one for each child
   }
@@ -232,16 +237,21 @@ BOOLEAN kVerify2(ideal F, ideal Q)
     pid = fork_process();
     if (pid==0) break; //child
   }
+  // input queue: queue: <index of L> -1 ...-1
+  // output queue: rqueue: pid .. pid 0 pid... pid for failure
+  //                      pid ... pid for success
   if (parent_pid!=getpid()) // child ------------------------------------------
   {
+    si_set_signal(SIGTERM,sig_term_hdl_child);
+    singular_close_links();
     loop
     {
       int ind=queue->dequeue();
-      if (ind== -1)
+      if (ind== -1) // negative number as stop sign
       {
         if (TEST_OPT_PROT) printf("child: end of queue\n");
-        rqueue->enqueue(0);
-        exit(0);
+        rqueue->enqueue(getpid()); // report pid of ending child
+        _exit(0);
       }
       int red_result=1;
       /* picks the element from the lazyset L */
@@ -283,42 +293,64 @@ BOOLEAN kVerify2(ideal F, ideal Q)
       if (red_result!=0)
       {
         if (TEST_OPT_PROT) printf("fail: result: %d\n",red_result);
-        rqueue->enqueue(1);
-        exit(0); // found fail, no need to test further
+        rqueue->enqueue(0);
+        rqueue->enqueue(getpid()); // pid of ending child
+        _exit(0); // found fail, no need to test further
       }
     }
-    exit(0); // all done, quit child
+    // should never be reached:
+    rqueue->enqueue(getpid()); // stop sign
+    _exit(0); // all done, quit child
   }
   else // parent ---------------------------------------------------
   {
     if (TEST_OPT_PROT) printf("%d children created\n",cpus);
     // wait for all process to stop:
-    // each process sends an 0 at end or a 1 for failure
+    // each process sends 0 for failure and its pid at end
+    // for success only the pid is send the end
     int res;
     int remaining_children=cpus;
     while(remaining_children>0)
     {
       res=rqueue->dequeue();
-      if (res==0) // a child finished
+      if (res>0) // a child finished
       {
         if (TEST_OPT_PROT) { printf("c");mflush(); }
-        //waitpid(-1,NULL,0); // ? see sig_chld_hdl
+        if (si_waitpid(res,NULL,WNOHANG)==0) // child not finished
+        {
+          if (kill(res,0)==0) // child is still running
+          {
+            struct timespec t;
+            struct timespec rem;
+            // wait till signal or 10s:
+            t.tv_sec=10;
+            t.tv_nsec=0;
+            nanosleep(&t, &rem); // should be interrupted by signal: SIG_CHLD
+            // child finished ?
+            if (si_waitpid(res,NULL,WNOHANG) ==0) //child not finished
+            {
+              kill(res,SIGTERM);
+              t.tv_sec=10;
+              t.tv_nsec=0;
+              nanosleep(&t, &rem); // should be interrupted by signal: SIG_CHLD
+              si_waitpid(res,NULL,WNOHANG);
+            }
+          }
+        }
         remaining_children--;
       }
-      else if (res==1) // not a GB - clean up and return 0
+      else if (res==0) // not a GB - clean up and return 0
       {
         if (TEST_OPT_PROT) { printf("C"); mflush(); }
-        remaining_children--;
         all_okay=FALSE;
         // clean queue:
         int dummy;
         do
         {
           dummy=queue->dequeue(); // remove remaining tasks
-        } while (dummy==0);
+        } while (dummy>=0);
       }
     }
-    sleep(1); // let all the children finish
     // removes queues
     queue.free();
     rqueue.free();
