@@ -90,9 +90,9 @@ static void syMinStep(ideal mod,ideal &syz,BOOLEAN final=FALSE,ideal up=NULL,
   if ((final) && (h==isHomog))
   /*minim is TRUE, we are in the module: maxlength, maxlength <>0*/
   {
-    ideal deg0=id_Jet(syz,0,currRing);
+    ideal deg0=id_Jet0(syz,currRing);
     id_Delete(&syz,currRing);
-    idSkipZeroes(deg0);
+    idSkipZeroes0(deg0);
     syz=deg0;
   }
 /*--cancels empty entees and their related components above--*/
@@ -220,24 +220,43 @@ void syGaussForOne(ideal syz, int elnum, int ModComp,int from,int till)
   int /*k,j,i,*/lu;
   poly unit1,unit2;
   poly actWith=syz->m[elnum];
+  syz->m[elnum] = NULL;
 
   if (from<0) from = 0;
   if ((till<=0) || (till>IDELEMS(syz))) till = IDELEMS(syz);
-  syz->m[elnum] = NULL;
   if (!rField_has_simple_inverse(currRing)) p_Cleardenom(actWith, currRing);
 /*--makes Gauss alg. for the column ModComp--*/
   pTakeOutComp(&(actWith), ModComp, &unit1, &lu);
-  while (from<till)
+  if (lu==1) /*p_IsConstantComp(unit1,currRing)*/
   {
-    poly tmp=syz->m[from];
-    if (/*syz->m[from]*/ tmp!=NULL)
+    number n_unit1=pGetCoeff(unit1);
+    while (from<till)
     {
-      pTakeOutComp(&(tmp), ModComp, &unit2, &lu);
-      tmp = pMult(pCopy(unit1),tmp);
-      syz->m[from] = pSub(tmp,
-        pMult(unit2,pCopy(actWith)));
+      poly tmp=syz->m[from];
+      if (/*syz->m[from]*/ tmp!=NULL)
+      {
+        pTakeOutComp(&tmp, ModComp, &unit2, &lu);
+        tmp = p_Mult_nn(tmp,n_unit1,currRing);
+        syz->m[from] = pSub(tmp,
+          pMult(unit2,pCopy(actWith)));
+      }
+      from++;
+    }
+  }
+  else
+  {
+    while (from<till)
+    {
+      poly tmp=syz->m[from];
+      if (/*syz->m[from]*/ tmp!=NULL)
+      {
+        pTakeOutComp(&tmp, ModComp, &unit2, &lu);
+        tmp = pMult(pCopy(unit1),tmp);
+        syz->m[from] = pSub(tmp,
+          pMult(unit2,pCopy(actWith)));
+      }
+      from++;
     }
-    from++;
   }
   pDelete(&actWith);
   pDelete(&unit1);
@@ -250,14 +269,8 @@ static void syDeleteAbove1(ideal up, int k)
     for (int i=0;i<IDELEMS(up);i++)
     {
       p = up->m[i];
-      while ((p!=NULL) && (pGetComp(p)==k))
+      while ((p!=NULL) && ((int)__p_GetComp(p,currRing)==k))
       {
-        /*
-        pp = pNext(p);
-        pNext(p) = NULL;
-        pDelete(&p);
-        p = pp;
-        */
         pLmDelete(&p);
       }
       up->m[i] = p;
@@ -265,14 +278,8 @@ static void syDeleteAbove1(ideal up, int k)
       {
         while (pNext(p)!=NULL)
         {
-          if (pGetComp(pNext(p))==k)
+          if ((int)__p_GetComp(pNext(p),currRing)==k)
           {
-            /*
-            pp = pNext(pNext(p));
-            pNext(pNext(p)) = NULL;
-            pDelete(&pNext(p));
-            pNext(p) = pp;
-            */
             pLmDelete(&pNext(p));
           }
           else
@@ -297,7 +304,7 @@ static void syMinStep1(resolvente res, int length)
 /*---we take out dependent elements from syz---------------------*/
     if (res[index+1]!=NULL)
     {
-      ideal deg0 = id_Jet(res[index+1],0,currRing);
+      ideal deg0 = id_Jet0(res[index+1],currRing);
       ideal reddeg0 = kInterRedOld(deg0);
       idDelete(&deg0);
       have_del = new intvec(IDELEMS(res[index]));
@@ -305,7 +312,7 @@ static void syMinStep1(resolvente res, int length)
       {
         if (reddeg0->m[i]!=NULL)
         {
-          j = pGetComp(reddeg0->m[i]);
+          j = (int)__p_GetComp(reddeg0->m[i],currRing);
           pDelete(&(res[index]->m[j-1]));
           /*res[index]->m[j-1] = NULL;*/
           (*have_del)[j-1] = 1;
@@ -325,15 +332,20 @@ static void syMinStep1(resolvente res, int length)
           while (k<IDELEMS(res[index]))
           {
             p = res[index]->m[k];
-            while ((p!=NULL) && ((!pLmIsConstantComp(p)) || (pGetComp(p)!=j)))
+            while ((p!=NULL)
+            && ((!pLmIsConstantComp(p)) || ((int)__p_GetComp(p,currRing)!=j)))
               pIter(p);
-            if ((p!=NULL) && (pLmIsConstantComp(p)) && (pGetComp(p)==j)) break;
+            if ((p!=NULL)
+            && (pLmIsConstantComp(p))
+            && ((int)__p_GetComp(p,currRing)==j)) break;
             k++;
           }
+          #ifndef SING_NDEBUG
           if (k>=IDELEMS(res[index]))
           {
             PrintS("out of range\n");
           }
+          #endif
           syGaussForOne(res[index],k,j);
           if (res[index+1]!=NULL)
             syDeleteAbove1(res[index+1],k+1);
@@ -624,7 +636,7 @@ syStrategy syResolution(ideal arg, int maxlength,intvec * w, BOOLEAN minim)
     }
     const unsigned int m_iFirstAltVar = scaFirstAltVar(currRing);
     const unsigned int m_iLastAltVar  = scaLastAltVar(currRing);
-    arg = id_KillSquares(arg, m_iFirstAltVar, m_iLastAltVar, currRing, false); // kill suares in input!
+    arg = id_KillSquares(idCopy(arg), m_iFirstAltVar, m_iLastAltVar, currRing, false); // kill squares in input!
   }
 #endif
 
@@ -842,9 +854,9 @@ intvec * syBetti(resolvente res,int length, int * regularity,
       if (res[i]->m[j]!=NULL)
       {
         if ((pGetComp(res[i]->m[j])>l)
-	// usual resolutions do not the following, but artifulal built may: (tr. #763)
+        // usual resolutions do not the following, but artifulal built may: (tr. #763)
         //|| ((i>1) && (res[i-1]->m[pGetComp(res[i]->m[j])-1]==NULL))
-	)
+        )
         {
           WerrorS("input not a resolution");
           omFreeSize((ADDRESS)temp1,(l+1)*sizeof(int));
