@@ -135,7 +135,6 @@ int  scDimInt(ideal S, ideal Q)
 
 int  scDimIntRing(ideal vid, ideal Q)
 {
-#ifdef HAVE_RINGS
   if (rField_is_Ring(currRing))
   {
     int i = idPosConstant(vid);
@@ -197,7 +196,6 @@ int  scDimIntRing(ideal vid, ideal Q)
     idDelete(&vv);
     return d;
   }
-#endif
   return scDimInt(vid,Q);
 }
 
@@ -923,30 +921,6 @@ void scPrintDegree(int co, int mu)
     Print("// dimension (local)   = %d\n// multiplicity = %d\n", di, mu);
 }
 
-void scDegree(ideal S, intvec *modulweight, ideal Q)
-{
-  id_Test(S, currRing);
-  if( Q!=NULL ) id_Test(Q, currRing);
-
-  int co, mu, l;
-  intvec *hseries2;
-  intvec *hseries1 = hFirstSeries(S, modulweight, Q);
-  if (errorreported) return;
-  l = hseries1->length()-1;
-  if (l > 1)
-    hseries2 = hSecondSeries(hseries1);
-  else
-    hseries2 = hseries1;
-  hDegreeSeries(hseries1, hseries2, &co, &mu);
-  if ((l == 1) &&(mu == 0))
-    scPrintDegree((currRing->N)+1, 0);
-  else
-    scPrintDegree(co, mu);
-  if (l>1)
-    delete hseries1;
-  delete hseries2;
-}
-
 long  scMult0Int(ideal S, ideal Q)
 {
   id_LmTest(S, currRing);
@@ -1104,7 +1078,6 @@ void scComputeHC(ideal S, ideal Q, int ak, poly &hEdge)
 
   int  i;
   int  k = ak;
-  #ifdef HAVE_RINGS
   if (rField_is_Ring(currRing) && (currRing->OrdSgn == -1))
   {
     //consider just monic generators (over rings with zero-divisors)
@@ -1129,7 +1102,6 @@ void scComputeHC(ideal S, ideal Q, int ak, poly &hEdge)
   }
   //getchar();
   #endif
-  #endif
   if(idElem(S) == 0)
     return;
   hNvar = (currRing->N);
@@ -1986,30 +1958,34 @@ static std::vector<std::vector<int> > iv2vv(intvec* M)
   return mat;
 }
 
+#if 0
 static void vvPrint(const std::vector<std::vector<int> >& mat)
 {
-  for (int i = 0; i < mat.size(); i++)
+  for (std::vector<std::vector<int> >::size_type i = 0; i < mat.size(); i++)
   {
-    for (int j = 0; j < mat[i].size(); j++)
+    for (std::vector<std::vector<int> >::size_type j = 0; j < mat[i].size(); j++)
     {
       Print("%d ", mat[i][j]);
     }
     PrintLn();
   }
 }
+#endif
 
+#if 0
 static void vvTest(const std::vector<std::vector<int> >& mat)
 {
   if (mat.size() > 0)
   {
-    int cols = mat[0].size();
-    for (int i = 1; i < mat.size(); i++)
+    std::vector<std::vector<int> >::size_type cols = mat[0].size();
+    for (std::vector<std::vector<int> >::size_type i = 1; i < mat.size(); i++)
     {
       if (cols != mat[i].size())
         WerrorS("number of cols in matrix inconsistent");
     }
   }
 }
+#endif
 
 static void vvDeleteRow(std::vector<std::vector<int> >& mat, int row)
 {
@@ -2018,7 +1994,7 @@ static void vvDeleteRow(std::vector<std::vector<int> >& mat, int row)
 
 static void vvDeleteColumn(std::vector<std::vector<int> >& mat, int col)
 {
-  for (int i = 0; i < mat.size(); i++)
+  for (std::vector<std::vector<int> >::size_type i = 0; i < mat.size(); i++)
   {
     mat[i].erase(mat[i].begin() + col);
   }
@@ -2026,7 +2002,7 @@ static void vvDeleteColumn(std::vector<std::vector<int> >& mat, int col)
 
 static BOOLEAN vvIsRowZero(const std::vector<std::vector<int> >& mat, int row)
 {
-  for (int i = 0; i < mat[row].size(); i++)
+  for (std::vector<std::vector<int> >::size_type i = 0; i < mat[row].size(); i++)
   {
     if (mat[row][i] != 0)
       return FALSE;
@@ -2036,7 +2012,7 @@ static BOOLEAN vvIsRowZero(const std::vector<std::vector<int> >& mat, int row)
 
 static BOOLEAN vvIsColumnZero(const std::vector<std::vector<int> >& mat, int col)
 {
-  for (int i = 0; i < mat.size(); i++)
+  for (std::vector<std::vector<int> >::size_type i = 0; i < mat.size(); i++)
   {
     if (mat[i][col] != 0)
       return FALSE;
@@ -2046,7 +2022,7 @@ static BOOLEAN vvIsColumnZero(const std::vector<std::vector<int> >& mat, int col
 
 static BOOLEAN vvIsZero(const std::vector<std::vector<int> >& mat)
 {
-  for (int i = 0; i < mat.size(); i++)
+  for (std::vector<std::vector<int> >::size_type i = 0; i < mat.size(); i++)
   {
     if (!vvIsRowZero(mat, i))
       return FALSE;
@@ -2056,10 +2032,10 @@ static BOOLEAN vvIsZero(const std::vector<std::vector<int> >& mat)
 
 static std::vector<std::vector<int> > vvMult(const std::vector<std::vector<int> >& a, const std::vector<std::vector<int> >& b)
 {
-  int ra = a.size();
-  int rb = b.size();
-  int ca = a.size() > 0 ? a[0].size() : 0;
-  int cb = b.size() > 0 ? b[0].size() : 0;
+  std::vector<std::vector<int> >::size_type ra = a.size();
+  std::vector<std::vector<int> >::size_type rb = b.size();
+  std::vector<std::vector<int> >::size_type ca = a.size() > 0 ? a[0].size() : 0;
+  std::vector<std::vector<int> >::size_type cb = b.size() > 0 ? b[0].size() : 0;
 
   if (ca != rb)
   {
@@ -2068,12 +2044,12 @@ static std::vector<std::vector<int> > vvMult(const std::vector<std::vector<int>
   }
 
   std::vector<std::vector<int> > res(ra, std::vector<int>(cb));
-  for (int i = 0; i < ra; i++)
+  for (std::vector<std::vector<int> >::size_type i = 0; i < ra; i++)
   {
-    for (int j = 0; j < cb; j++)
+    for (std::vector<std::vector<int> >::size_type j = 0; j < cb; j++)
     {
       int sum = 0;
-      for (int k = 0; k < ca; k++)
+      for (std::vector<std::vector<int> >::size_type k = 0; k < ca; k++)
         sum += a[i][k] * b[k][j];
       res[i][j] = sum;
     }
@@ -2242,7 +2218,7 @@ int lp_kDim(const ideal _G)
   }
 
   std::vector<std::vector<int> > vvUG = iv2vv(UG);
-  for (int i = 0; i < vvUG.size(); i++)
+  for (std::vector<std::vector<int> >::size_type i = 0; i < vvUG.size(); i++)
   {
     if (vvIsRowZero(vvUG, i) && vvIsColumnZero(vvUG, i)) // i is isolated vertex
     {
@@ -2264,9 +2240,9 @@ int lp_kDim(const ideal _G)
   {
     if (TEST_OPT_PROT)
       PrintS("Start count graph entries.\n");
-    for (int i = 0; i < UGpower.size(); i++)
+    for (std::vector<std::vector<int> >::size_type i = 0; i < UGpower.size(); i++)
     {
-      for (int j = 0; j < UGpower[i].size(); j++)
+      for (std::vector<std::vector<int> >::size_type j = 0; j < UGpower[i].size(); j++)
       {
         numberOfNormalWords += UGpower[i][j];
       }
