@@ -197,12 +197,12 @@ static ideal idSectWithElim (ideal h1,ideal h2, GbVariant alg)
   return res;
 }
 
-static ideal idGroebner(ideal temp,int syzComp,GbVariant alg, intvec* hilb=NULL, intvec* w=NULL, tHomog hom=testHomog)
+static ideal idGroebner(ideal temp,int syzComp,GbVariant alg, bigintmat* hilb=NULL, intvec* w=NULL, tHomog hom=testHomog)
 {
   //Print("syz=%d\n",syzComp);
   //PrintS(showOption());
   //PrintLn();
-  ideal temp1;
+  ideal res=NULL;
   if (w==NULL)
   {
     if (hom==testHomog)
@@ -219,24 +219,24 @@ static ideal idGroebner(ideal temp,int syzComp,GbVariant alg, intvec* hilb=NULL,
   if ((alg==GbStd)||(alg==GbDefault))
   {
     if (TEST_OPT_PROT &&(alg==GbStd)) { PrintS("std:"); mflush(); }
-    temp1 = kStd(temp,currRing->qideal,hom,&w,hilb,syzComp);
+    res = kStd(temp,currRing->qideal,hom,&w,hilb,syzComp);
     idDelete(&temp);
   }
   else if (alg==GbSlimgb)
   {
     if (TEST_OPT_PROT) { PrintS("slimgb:"); mflush(); }
-    temp1 = t_rep_gb(currRing, temp, syzComp);
+    res = t_rep_gb(currRing, temp, syzComp);
     idDelete(&temp);
   }
   else if (alg==GbGroebner)
   {
     if (TEST_OPT_PROT) { PrintS("groebner:"); mflush(); }
     BOOLEAN err;
-    temp1=(ideal)iiCallLibProc1("groebner",temp,MODUL_CMD,err);
+    res=(ideal)iiCallLibProc1("groebner",temp,MODUL_CMD,err);
     if (err)
     {
       Werror("error %d in >>groebner<<",err);
-      temp1=idInit(1,1);
+      res=idInit(1,1);
     }
   }
   else if (alg==GbModstd)
@@ -246,18 +246,18 @@ static ideal idGroebner(ideal temp,int syzComp,GbVariant alg, intvec* hilb=NULL,
     void *args[]={temp,(void*)1,NULL};
     int arg_t[]={MODUL_CMD,INT_CMD,0};
     leftv temp0=ii_CallLibProcM("modStd",args,arg_t,currRing,err);
-    temp1=(ideal)temp0->data;
+    res=(ideal)temp0->data;
     omFreeBin((ADDRESS)temp0,sleftv_bin);
     if (err)
     {
       Werror("error %d in >>modStd<<",err);
-      temp1=idInit(1,1);
+      res=idInit(1,1);
     }
   }
   else if (alg==GbSba)
   {
     if (TEST_OPT_PROT) { PrintS("sba:"); mflush(); }
-    temp1 = kSba(temp,currRing->qideal,hom,&w,1,0,NULL);
+    res = kSba(temp,currRing->qideal,hom,&w,1,0,NULL);
     if (w!=NULL) delete w;
   }
   else if (alg==GbStdSat)
@@ -296,17 +296,17 @@ static ideal idGroebner(ideal temp,int syzComp,GbVariant alg, intvec* hilb=NULL,
       void *args[]={temp,v,NULL};
       int arg_t[]={MODUL_CMD,IDEAL_CMD,0};
       leftv temp0=ii_CallLibProcM("satstd",args,arg_t,currRing,err);
-      temp1=(ideal)temp0->data;
+      res=(ideal)temp0->data;
       omFreeBin((ADDRESS)temp0, sleftv_bin);
     }
     if (err)
     {
       Werror("error %d in >>satstd<<",err);
-      temp1=idInit(1,1);
+      res=idInit(1,1);
     }
   }
   if (w!=NULL) delete w;
-  return temp1;
+  return res;
 }
 
 /*2
@@ -356,6 +356,7 @@ ideal idSect (ideal h1,ideal h2, GbVariant alg)
   ring syz_ring=rAssure_SyzOrder(orig_ring,TRUE);
   rSetSyzComp(length,syz_ring);
   rChangeCurrRing(syz_ring);
+  si_opt_1 |= Sy_bit(OPT_REDTAIL_SYZ);
 
   while ((j>0) && (first->m[j-1]==NULL)) j--;
   temp = idInit(j /*IDELEMS(first)*/+IDELEMS(second),length+j);
@@ -503,6 +504,10 @@ ideal idMultSect(resolvente arg, int length, GbVariant alg)
   j += maxrk;
   syzComp = k*maxrk;
 
+  BITSET save_opt;
+  SI_SAVE_OPT1(save_opt);
+  si_opt_1 |= Sy_bit(OPT_REDTAIL_SYZ);
+
   ring orig_ring=currRing;
   ring syz_ring=rAssure_SyzOrder(orig_ring,TRUE);
   rSetSyzComp(syzComp,syz_ring);
@@ -582,6 +587,7 @@ ideal idMultSect(resolvente arg, int length, GbVariant alg)
     rChangeCurrRing(orig_ring);
     rDelete(syz_ring);
   }
+  SI_RESTORE_OPT1(save_opt);
   idSkipZeroes(result);
   return result;
 }
@@ -993,9 +999,9 @@ ideal idLiftStd (ideal  h1, matrix* T, tHomog hi, ideal * S, GbVariant alg,
     return idInit(1,h1->rank);
   }
 
-  BITSET save2;
-  SI_SAVE_OPT2(save2);
-
+  BITSET saveOpt1,saveOpt2;
+  SI_SAVE_OPT(saveOpt1,saveOpt2);
+  si_opt_2|=Sy_bit(V_NOT_TRICKS);
   k=si_max(1,inputIsIdeal);
 
   if ((!lift3)&&(!TEST_OPT_RETURN_SB)) si_opt_2 |=Sy_bit(V_IDLIFT);
@@ -1034,7 +1040,7 @@ ideal idLiftStd (ideal  h1, matrix* T, tHomog hi, ideal * S, GbVariant alg,
 
   if (syz_ring!=orig_ring) rDelete(syz_ring);
   s_h3->rank=h1->rank;
-  SI_RESTORE_OPT2(save2);
+  SI_RESTORE_OPT(saveOpt1,saveOpt2);
   return s_h3;
 }
 
@@ -1602,7 +1608,7 @@ ideal idQuot (ideal  h1, ideal h2, BOOLEAN h1IsStb, BOOLEAN resultIsIdeal)
 /*2
 * eliminate delVar (product of vars) in h1
 */
-ideal idElimination (ideal h1,poly delVar,intvec *hilb, GbVariant alg)
+ideal idElimination (ideal h1,poly delVar,bigintmat *hilb, GbVariant alg)
 {
   int    i,j=0,k,l;
   ideal  h,hh, h3;
@@ -2324,14 +2330,12 @@ ideal idModuloLP (ideal h2,ideal h1, tHomog, intvec ** w, matrix *T, GbVariant a
   }
 
   idTest(s_temp);
-  unsigned save_opt,save_opt2;
-  SI_SAVE_OPT1(save_opt);
-  SI_SAVE_OPT2(save_opt2);
+  BITSET save_opt,save_opt2;
+  SI_SAVE_OPT(save_opt,save_opt2);
   if (T==NULL) si_opt_1 |= Sy_bit(OPT_REDTAIL_SYZ);
   si_opt_1 |= Sy_bit(OPT_REDTAIL);
   ideal s_temp1 = idGroebner(s_temp,length,alg);
-  SI_RESTORE_OPT1(save_opt);
-  SI_RESTORE_OPT2(save_opt2);
+  SI_RESTORE_OPT(save_opt,save_opt2);
 
   //if (wtmp!=NULL)  Print("output weights:");wtmp->show(1);PrintLn();
   if ((w!=NULL) && (*w !=NULL) && (wtmp!=NULL))
@@ -2483,14 +2487,12 @@ ideal idModulo (ideal h2,ideal h1, tHomog hom, intvec ** w, matrix *T, GbVariant
       s2=idCopy(h2);
     }
 
-    unsigned save_opt,save_opt2;
-    SI_SAVE_OPT1(save_opt);
-    SI_SAVE_OPT2(save_opt2);
+    BITSET save_opt,save_opt2;
+    SI_SAVE_OPT(save_opt,save_opt2);
     if (T==NULL) si_opt_1 |= Sy_bit(OPT_REDTAIL);
     si_opt_1 |= Sy_bit(OPT_REDTAIL_SYZ);
     s_temp1 = idPrepare(s2,s1,testHomog,length,w,alg);
-    SI_RESTORE_OPT1(save_opt);
-    SI_RESTORE_OPT2(save_opt2);
+    SI_RESTORE_OPT(save_opt,save_opt2);
   }
 
   //if (wtmp!=NULL)  Print("output weights:");wtmp->show(1);PrintLn();
@@ -2688,21 +2690,103 @@ static void idDeleteComps(ideal arg,int* red_comp,int del)
   (arg->rank) -= del;
 }
 
+/*3
+* searches for the next unit in the components of the module arg and
+* returns the first one;
+*/
+static int id_ReadOutPivot(ideal arg,int* comp, const ring r)
+{
+  int i=0,j, generator=-1;
+  int rk_arg=arg->rank; //idRankFreeModule(arg);
+  int * componentIsUsed =(int *)omAlloc((rk_arg+1)*sizeof(int));
+  poly p;
+
+  while ((generator<0) && (i<IDELEMS(arg)))
+  {
+    memset(componentIsUsed,0,(rk_arg+1)*sizeof(int));
+    p = arg->m[i];
+    if (rField_is_Ring(r))
+    {
+      while (p!=NULL)
+      {
+        j = __p_GetComp(p,r);
+        if (componentIsUsed[j]==0)
+        {
+          if (p_LmIsConstantComp(p,r) &&
+              n_IsUnit(pGetCoeff(p),r->cf))
+          {
+            generator = i;
+            componentIsUsed[j] = 1;
+          }
+          else
+          {
+            componentIsUsed[j] = -1;
+          }
+        }
+        else if (componentIsUsed[j]>0)
+        {
+          (componentIsUsed[j])++;
+        }
+        pIter(p);
+      }
+    }
+    else
+    {
+      while (p!=NULL)
+      {
+        j = __p_GetComp(p,r);
+        if (componentIsUsed[j]==0)
+        {
+          if (p_LmIsConstantComp(p,r))
+          {
+            generator = i;
+            componentIsUsed[j] = 1;
+          }
+          else
+          {
+            componentIsUsed[j] = -1;
+          }
+        }
+        else if (componentIsUsed[j]>0)
+        {
+          (componentIsUsed[j])++;
+        }
+        pIter(p);
+      }
+    }
+    i++;
+  }
+  i = 0;
+  *comp = -1;
+  for (j=0;j<=rk_arg;j++)
+  {
+    if (componentIsUsed[j]>0)
+    {
+      if ((*comp==-1) || (componentIsUsed[j]<i))
+      {
+        *comp = j;
+        i= componentIsUsed[j];
+      }
+    }
+  }
+  omFree(componentIsUsed);
+  return generator;
+}
+
 /*2
 * returns the presentation of an isomorphic, minimally
 * embedded  module (arg represents the quotient!)
 */
-ideal idMinEmbedding(ideal arg,BOOLEAN inPlace, intvec **w)
+static ideal idMinEmbedding1(ideal arg,BOOLEAN inPlace, intvec **w,
+  int* red_comp, int &del)
 {
   if (idIs0(arg)) return idInit(1,arg->rank);
   int i,next_gen,next_comp;
   ideal res=arg;
   if (!inPlace) res = idCopy(arg);
   res->rank=si_max(res->rank,id_RankFreeModule(res,currRing));
-  int *red_comp=(int*)omAlloc((res->rank+1)*sizeof(int));
   for (i=res->rank;i>=0;i--) red_comp[i]=i;
 
-  int del=0;
   loop
   {
     next_gen = id_ReadOutPivot(res, &next_comp, currRing);
@@ -2716,21 +2800,147 @@ ideal idMinEmbedding(ideal arg,BOOLEAN inPlace, intvec **w)
     }
   }
 
-  idDeleteComps(res,red_comp,del);
   idSkipZeroes(res);
-  omFree(red_comp);
 
   if ((w !=NULL)&&(*w!=NULL) &&(del>0))
   {
     int nl=si_max((*w)->length()-del,1);
     intvec *wtmp=new intvec(nl);
-    for(i=0;i<res->rank;i++) (*wtmp)[i]=(**w)[i];
+    for(i=0;i<nl;i++) (*wtmp)[i]=(**w)[i];
     delete *w;
     *w=wtmp;
   }
   return res;
 }
 
+ideal idMinEmbedding(ideal arg,BOOLEAN inPlace, intvec **w)
+{
+  int *red_comp=(int*)omAlloc((arg->rank+1)*sizeof(int));
+  int del=0;
+  ideal res=idMinEmbedding1(arg,inPlace,w,red_comp,del);
+  idDeleteComps(res,red_comp,del);
+  omFree(red_comp);
+  return res;
+}
+
+ideal idMinEmbedding_with_map(ideal arg,intvec **w, ideal &trans)
+{
+  int *red_comp=(int*)omAlloc((arg->rank+1)*sizeof(int));
+  int del=0;
+  ideal res=idMinEmbedding1(arg,FALSE,w,red_comp,del);
+  trans=idLift(arg,res,NULL,TRUE,FALSE,FALSE,NULL);
+  //idDeleteComps(res,red_comp,del);
+  omFree(red_comp);
+  return res;
+}
+
+ideal idMinEmbedding_with_map_v(ideal arg,intvec **w, ideal &trans, int* g)
+{
+  if (idIs0(arg))
+  {
+    trans=idFreeModule(arg->rank);
+    if (g!=NULL)
+    {
+      for(int i=0;i<arg->rank;i++) g[i]=i+1;
+    }
+    return arg;
+  }
+  int *red_comp=(int*)omAlloc((arg->rank+1)*sizeof(int));
+  int del=0;
+  ideal res=idMinEmbedding1(arg,FALSE,w,red_comp,del);
+  trans=idLift(arg,res,NULL,TRUE,FALSE,FALSE,NULL);
+  for(int i=1;i<=arg->rank;i++)
+  {
+    g[i-1]=red_comp[i];
+  }
+  idDeleteComps(res,red_comp,del);
+  return res;
+}
+
+extern void ipPrint_MA0(matrix m, const char *name);
+#if 0 // unused
+ideal idMinEmbedding_with_map0(ideal arg,intvec **w, ideal &trans)
+{
+  ideal a=idCopy(arg);
+  // add unit matrix to a
+  int k=a->rank+1;
+  const int rk=a->rank;
+  poly p;
+  int i;
+  for(i=0;i<IDELEMS(a);i++,k++)
+  {
+    p=pOne();
+    pSetComp(p,k);pSetmComp(p);
+    a->m[i]=p_Add_q(a->m[i],p,currRing);
+  }
+  // search a unit in orig part of a
+  // and subtract, start at start
+  int start=0;
+  loop
+  {
+  PrintS("matrix:\n");
+  ipPrint_MA0((matrix)a,"a");
+    i=start;
+    if (i>=IDELEMS(a)) break;
+    p=a->m[i];
+    start=IDELEMS(a);
+    while((p!=NULL)&&(pGetComp(p)<=rk)&&(p_Totaldegree(p,currRing)>0)) pIter(p);
+    if ((p!=NULL)&&(pGetComp(p)<=rk)&&(p_Totaldegree(p,currRing)==0))
+    { // found const in vector i, comp k
+      k=pGetComp(p);
+      // normalize:
+      number n=nCopy(pGetCoeff(p));
+      n=nInpNeg(n);
+      a->m[i]=p_Div_nn(p,n,currRing);
+      // subtract
+      BOOLEAN changed=FALSE;
+      for(int j=IDELEMS(a)-1;j>=0;j--)
+      {
+        if (j!=i)
+        {
+          poly q=p_Vec2Poly(a->m[j],k,currRing);
+          if (q!=NULL)
+          {
+            start=j; // changed entries start at j
+            changed=TRUE;
+            poly s=p_Mult_q(a->m[i],q,currRing);
+            a->m[j]=p_Add_q(a->m[j],s,currRing);
+          }
+        }
+      }
+      if(changed) continue;
+    }
+    else i++;
+  }
+  // a -> result,trans
+  trans=idInit(IDELEMS(a),IDELEMS(a));
+  ideal result=idInit(IDELEMS(a),rk);
+  for(i=0;i<IDELEMS(a);i++)
+  {
+    while(a->m[i]!=NULL)
+    {
+      poly p=a->m[i];
+      a->m[i]=p->next;
+      p->next=NULL;
+      if(pGetComp(p)<=rk)
+      {
+        result->m[i]=p_Add_q(result->m[i],p,currRing);
+      }
+      else
+      {
+        p_Shift(&p,-rk,currRing);
+        trans->m[i]=p_Add_q(trans->m[i],p,currRing);
+      }
+    }
+  }
+  PrintS("prune:\n");
+  ipPrint_MA0((matrix)result,"R");
+  PrintS("trans:\n");
+  ipPrint_MA0((matrix)trans,"T");
+  idDelete(&a);
+  return result;
+}
+#endif
 #include "polys/clapsing.h"
 
 #if 0
@@ -3245,14 +3455,24 @@ ideal id_Sat_principal(ideal I, ideal J, const ring origR)
   return TTT;
 }
 
-ideal idSaturate(ideal I, ideal J, int &k, BOOLEAN isIdeal)
+ideal idSaturate_intern(ideal I, ideal J, int &k, BOOLEAN isIdeal, BOOLEAN isSB)
 {
-  if(idIs0(I))
+  if(idIs0(J))
   {
-     I=idCopy(I);
-     idSkipZeroes(I);
-     return(I);
+     ideal res;
+     if(isIdeal)
+     {
+       res=idInit(1,1);
+       res->m[0]=pOne();
+     }
+     else
+     {
+       res=idFreeModule(I->rank);
+     }
+     k=1;
+     return(res);
   }
+  BITSET save_opt;SI_SAVE_OPT2(save_opt);
   //if (idElem(J)==1)
   //{
   //  idSkipZeroes(J);
@@ -3281,6 +3501,7 @@ ideal idSaturate(ideal I, ideal J, int &k, BOOLEAN isIdeal)
     ideal Iquot,Istd;
     intvec *w=NULL;
     Istd=id_Satstd(I,J,currRing);
+    si_opt_2|=Sy_bit(V_NOT_TRICKS);
     k=0;
     loop
     {
@@ -3304,6 +3525,7 @@ ideal idSaturate(ideal I, ideal J, int &k, BOOLEAN isIdeal)
   //iiWriteMatrix((matrix)J,"J",1,currRing,0); PrintLn();
   //iiWriteMatrix((matrix)Istd,"res",1,currRing,0);PrintLn();
   //id_Delete(&Istd,currRing);
+    SI_RESTORE_OPT2(save_opt);
     return Istd;
   }
   //--------------------------------------------------
@@ -3314,7 +3536,9 @@ ideal idSaturate(ideal I, ideal J, int &k, BOOLEAN isIdeal)
   loop
   {
     k++;
-    Iquot=idQuot(Istd,J,FALSE,isIdeal);
+    Iquot=idQuot(Istd,J,isSB,isIdeal);
+    isSB=FALSE;
+    si_opt_2|=Sy_bit(V_NOT_TRICKS); // used from 2nd loop on
     ideal tmp=kNF(Istd,currRing->qideal,Iquot,5);
     int  elem=idElem(tmp);
     id_Delete(&tmp,currRing);
@@ -3325,17 +3549,26 @@ ideal idSaturate(ideal I, ideal J, int &k, BOOLEAN isIdeal)
   k--;
   Istd=kStd(Iquot,currRing->qideal,testHomog,&w);
   idSkipZeroes(Istd);
+  SI_RESTORE_OPT2(save_opt);
   //if (only_vars)
   //{
   //  iiWriteMatrix((matrix)Istd,"org",1,currRing,0);
   //}
   return Istd;
 }
+ideal idSaturate(ideal I, ideal J, int &k, BOOLEAN isIdeal)
+{
+ return idSaturate_intern(I,J,k,isIdeal,FALSE);
+}
+ideal idSaturateGB(ideal I, ideal J, int &k, BOOLEAN isIdeal)
+{
+ return idSaturate_intern(I,J,k,isIdeal,TRUE);
+}
 
 ideal id_Homogenize(ideal I, int var_num, const ring r)
 {
   ideal II=id_Copy(I,r);
-  if (var_num=1)
+  if (var_num==1)
   {
     ring tmpR=rAssure_Dp_C(r);
     if (tmpR!=r)
@@ -3380,7 +3613,7 @@ ideal id_Homogenize(ideal I, int var_num, const ring r)
 ideal id_HomogenizeW(ideal I, int var_num, intvec *w,const ring r)
 {
   ideal II=id_Copy(I,r);
-  if (var_num=1)
+  if (var_num==1)
   {
     ring tmpR=rAssure_Wp_C(r,w);
     if (tmpR!=r)
