@@ -17,9 +17,9 @@
 #endif
 
 #define SINGULAR_MAJOR_VERSION 4
-#define SINGULAR_MINOR_VERSION 3
-#define SINGULAR_SUB_VERSION 2
-#define SINGULAR_PATCHLEVEL 9
+#define SINGULAR_MINOR_VERSION 4
+#define SINGULAR_SUB_VERSION 0
+#define SINGULAR_PATCHLEVEL 8
 #define S_ROOT_DIR ""
 
 /*******************************************************************
@@ -61,9 +61,9 @@
 
 /* define for parallel processes with shared memory */
 #ifndef __CCYGWIN__
-#if !defined(__MACH__) || !defined(__APPLE__)
-#define HAVE_VSPACE 1
-#endif
+  #if (!defined(__APPLE__))
+    #define HAVE_VSPACE 1
+  #endif
 #endif
 
 /*#define PROFILING*/
@@ -84,7 +84,7 @@
 #define ALIGN_8
 #endif
 
-#define SINGULAR_VERSION ((SINGULAR_MAJOR_VERSION*1000 + SINGULAR_MINOR_VERSION*100 + SINGULAR_SUB_VERSION*10)+SINGULAR_PATCHLEVEL)
+#define SINGULAR_VERSION ((SINGULAR_MAJOR_VERSION*10000 + SINGULAR_MINOR_VERSION*1000 + SINGULAR_SUB_VERSION*100)+SINGULAR_PATCHLEVEL)
 
 /*******************************************************************
  * Miscellaneous Defines
