@@ -5,7 +5,11 @@
 #ifdef HAVE_CPP_THREADS
 #include <thread>
 #endif
+#ifdef HAVE_POLL
+#include <poll.h>
+#endif
 #include <cstddef>
+#include "reporter/si_signals.h"
 
 #if defined(__GNUC__) && (__GNUC__<9) &&!defined(__clang__)
 
@@ -386,11 +390,53 @@ ipc_signal_t check_signal(bool resume, bool lock) {
       char buf[1];
       if (lock && sigstate == Waiting) {
         unlock_process(vmem.current_process);
-        while (read(fd, buf, 1) != 1) {
+        loop
+        {
+          #if defined(HAVE_POLL)  && !defined(__APPLE__)
+          // fd is restricted on OsX by ulimit "file descriptors" (256)
+          pollfd pfd;
+          pfd.fd = fd;
+          pfd.events = POLLIN;
+          int rv = poll(&pfd, 1, 10000); /* msec*/
+          #else
+          // fd is restricted to <=1024
+          fd_set set;
+          FD_ZERO(&set); /* clear the set */
+          FD_SET(fd, &set); /* add our file descriptor to the set */
+          struct timeval timeout;
+          timeout.tv_sec = 10;
+          timeout.tv_usec = 0;
+          int rv = si_select(fd + 1, &set, NULL, NULL, &timeout);
+          #endif
+          if (rv== -1) continue; /* an error occurred */
+          if (rv== 0) break;  /* timeout */
+          //while(read(fd, buf, 1)!=1) {}
+          if(read(fd, buf, 1)==1) break;
         }
         lock_process(vmem.current_process);
       } else {
-        while (read(fd, buf, 1) != 1) {
+        loop
+        {
+          #if defined(HAVE_POLL)  && !defined(__APPLE__)
+          // fd is restricted on OsX by ulimit "file descriptors" (256)
+          pollfd pfd;
+          pfd.fd = fd;
+          pfd.events = POLLIN;
+          int rv = poll(&pfd, 1, 10000); /* msec*/
+          #else
+          // fd is restricted to <=1024
+          fd_set set;
+          FD_ZERO(&set); /* clear the set */
+          FD_SET(fd, &set); /* add our file descriptor to the set */
+          struct timeval timeout;
+          timeout.tv_sec = 10;
+          timeout.tv_usec = 0;
+          int rv = si_select(fd + 1, &set, NULL, NULL, &timeout);
+          #endif
+          if (rv== -1) continue;  /* an error occurred */
+          if (rv== 0) break;  /* timeout */
+          //while(read(fd, buf, 1)!=1) {}
+          if(read(fd, buf, 1)==1) break;
         }
       }
       result = process_info(vmem.current_process).signal;
@@ -953,11 +999,53 @@ ipc_signal_t check_signal(bool resume, bool lock) {
       char buf[1];
       if (lock && sigstate == Waiting) {
         unlock_process(vmem.current_process);
-        while (read(fd, buf, 1) != 1) {
+        loop
+        {
+          #if defined(HAVE_POLL)  && !defined(__APPLE__)
+          // fd is restricted on OsX by ulimit "file descriptors" (256)
+          pollfd pfd;
+          pfd.fd = fd;
+          pfd.events = POLLIN;
+          int rv = poll(&pfd, 1, 10000); /* msec*/
+          #else
+          // fd is restricted to <=1024
+          fd_set set;
+          FD_ZERO(&set); /* clear the set */
+          FD_SET(fd, &set); /* add our file descriptor to the set */
+          struct timeval timeout;
+          timeout.tv_sec = 10;
+          timeout.tv_usec = 0;
+          int rv = si_select(fd + 1, &set, NULL, NULL, &timeout);
+          #endif
+          if (rv== -1) continue;  /* an error occurred */
+          if (rv== 0) break;  /* timeout */
+          //while(read(fd, buf, 1)!=1) {}
+          if(read(fd, buf, 1)==1) break;
         }
         lock_process(vmem.current_process);
       } else {
-        while (read(fd, buf, 1) != 1) {
+        loop
+        {
+          #if defined(HAVE_POLL)  && !defined(__APPLE__)
+          // fd is restricted on OsX by ulimit "file descriptors" (256)
+          pollfd pfd;
+          pfd.fd = fd;
+          pfd.events = POLLIN;
+          int rv = poll(&pfd, 1, 10000); /* msec*/
+          #else
+          // fd is restricted to <=1024
+          fd_set set;
+          FD_ZERO(&set); /* clear the set */
+          FD_SET(fd, &set); /* add our file descriptor to the set */
+          struct timeval timeout;
+          timeout.tv_sec = 10;
+          timeout.tv_usec = 0;
+          int rv = si_select(fd + 1, &set, NULL, NULL, &timeout);
+          #endif
+          if (rv== -1) continue; /* an error occurred */
+          if (rv== 0) break;/* timeout */
+          //while(read(fd, buf, 1)!=1) {}
+          if(read(fd, buf, 1)==1) break;
         }
       }
       result = process_info(vmem.current_process).signal;
