@@ -14,6 +14,8 @@ VAR ring  currRing = NULL;
 
 void rChangeCurrRing(ring r)
 {
+  if (currRing!=NULL)
+    currRing->options = si_opt_1 & TEST_RINGDEP_OPTS;
   //------------ set global ring vars --------------------------------
   currRing = r;
   if( r != NULL )
@@ -71,7 +73,7 @@ poly p_Divide(poly p, poly q, const ring r)
         ideal R; matrix U;
         ring save_ring=currRing;
         if (r!=currRing) rChangeCurrRing(r);
-        int save_opt;
+        BITSET save_opt;
         SI_SAVE_OPT1(save_opt);
         si_opt_1 &= ~(Sy_bit(OPT_PROT));
         ideal m = idLift(vi,ui,&R, FALSE,TRUE,TRUE,&U);
@@ -129,7 +131,7 @@ poly p_Divide(poly p, poly q, const ring r)
             ideal R; matrix U;
             ring save_ring=currRing;
             if (r!=currRing) rChangeCurrRing(r);
-            int save_opt;
+            BITSET save_opt;
             SI_SAVE_OPT1(save_opt);
             si_opt_1 &= ~(Sy_bit(OPT_PROT));
             ideal m = idLift(vi,ui,&R, FALSE,TRUE,TRUE,&U);
@@ -208,7 +210,7 @@ poly pp_Divide(poly p, poly q, const ring r)
         ideal R; matrix U;
         ring save_ring=currRing;
         if (r!=currRing) rChangeCurrRing(r);
-        int save_opt;
+        BITSET save_opt;
         SI_SAVE_OPT1(save_opt);
         si_opt_1 &= ~(Sy_bit(OPT_PROT));
         ideal m = idLift(vi,ui,&R, FALSE,TRUE,TRUE,&U);
@@ -268,7 +270,7 @@ poly pp_Divide(poly p, poly q, const ring r)
             ideal R; matrix U;
             ring save_ring=currRing;
             if (r!=currRing) rChangeCurrRing(r);
-            int save_opt;
+            BITSET save_opt;
             SI_SAVE_OPT1(save_opt);
             si_opt_1 &= ~(Sy_bit(OPT_PROT));
             ideal m = idLift(vi,ui,&R, FALSE,TRUE,TRUE,&U);
@@ -355,7 +357,7 @@ poly p_DivRem(poly p, poly q, poly &rest, const ring r)
       ideal R; matrix U;
       ring save_ring=currRing;
       if (r!=currRing) rChangeCurrRing(r);
-      int save_opt;
+      BITSET save_opt;
       SI_SAVE_OPT1(save_opt);
       si_opt_1 &= ~(Sy_bit(OPT_PROT));
       ideal m = idLift(vi,ui,&R, FALSE,TRUE,TRUE,&U);
@@ -413,7 +415,7 @@ poly singclap_gcd ( poly f, poly g, const ring r )
     intvec *w=NULL;
     ring save_ring=currRing;
     if (r!=currRing) rChangeCurrRing(r);
-    int save_opt;
+    BITSET save_opt;
     SI_SAVE_OPT1(save_opt);
     si_opt_1 &= ~(Sy_bit(OPT_PROT));
     ideal S1=idSyzygies(I,testHomog,&w);
