@@ -45,7 +45,14 @@ enum n_coeffType
   n_Znm, /**< only used if HAVE_RINGS is defined */
   n_Z2m, /**< only used if HAVE_RINGS is defined */
   n_FlintQrat, /**< rational funtion field over Q */
-  n_CF /**< ? */
+  n_CF, /**< ? */
+  n_Nemo_AnticNumberField, /*17 */
+  n_Nemo_QQField,          /*18 */
+  n_Nemo_ZZRing,           /*19*/
+  n_Nemo_FqPolyRepField,   /*20 */
+  n_Nemo_fqPolyRepField,   /*21 */
+  n_Nemo_Field,            /*22 */
+  n_Nemo_Ring              /*23 */
 };
 
 extern const unsigned short fftable[];
@@ -130,7 +137,7 @@ struct n_Procs_s
    int     factoryVarOffset;
 
    // general properties:
-   /// TRUE, if nNew/nDelete/nCopy are dummies
+   /// TRUE, if nDelete/nCopy are dummies
    BOOLEAN has_simple_Alloc;
    /// TRUE, if std should make polynomials monic (if nInvers is cheap)
    /// if false, then a gcd routine is used for a content computation
@@ -227,7 +234,7 @@ struct n_Procs_s
            (*cfEqual)(number a,number b, const coeffs r),
            (*cfIsZero)(number a, const coeffs r),
            (*cfIsOne)(number a, const coeffs r),
-       // IsMOne is used for printing os polynomials:
+       // IsMOne is used for printing of polynomials:
        // -1 is only printed for constant monomials
            (*cfIsMOne)(number a, const coeffs r),
        //GreaterZero is used for printing of polynomials:
@@ -377,7 +384,8 @@ struct n_Procs_s
    int     (*cfDivComp)(number a,number b,const coeffs r);
    BOOLEAN (*cfIsUnit)(number a,const coeffs r);
    number  (*cfGetUnit)(number a,const coeffs r);
-   //CF: test if b divides a
+   /// test if b divides a
+   /// cfDivBy(zero,b,r) is true, if b is a zero divisor
    BOOLEAN (*cfDivBy)(number a, number b, const coeffs r);
   /* The following members are for representing the ring Z/n,
      where n is not a prime. We distinguish four cases:
@@ -436,10 +444,6 @@ void nKillChar(coeffs r);
 static FORCE_INLINE void nSetChar(const coeffs r)
 { assume(r!=NULL); assume(r->cfSetChar != NULL); r->cfSetChar(r); }
 
-void           nNew(number * a);
-#define n_New(n, r)           nNew(n)
-
-
 /// Return the characteristic of the coeff. domain.
 static FORCE_INLINE int n_GetChar(const coeffs r)
 { assume(r != NULL); return r->ch; }
@@ -518,7 +522,6 @@ static FORCE_INLINE BOOLEAN n_IsUnit(number n, const coeffs r)
 static FORCE_INLINE coeffs n_CoeffRingQuot1(number c, const coeffs r)
 { assume(r != NULL); assume(r->cfQuot1 != NULL); return r->cfQuot1(c, r); }
 
-#ifdef HAVE_RINGS
 static FORCE_INLINE int n_DivComp(number a, number b, const coeffs r)
 { assume(r != NULL); assume(r->cfDivComp!=NULL); return r->cfDivComp (a,b,r); }
 
@@ -532,8 +535,6 @@ static FORCE_INLINE int n_DivComp(number a, number b, const coeffs r)
 static FORCE_INLINE number n_GetUnit(number n, const coeffs r)
 { assume(r != NULL); assume(r->cfGetUnit!=NULL); return r->cfGetUnit(n,r); }
 
-#endif
-
 /// a number representing i in the given coeff field/ring r
 static FORCE_INLINE number n_Init(long i,       const coeffs r)
 { assume(r != NULL); assume(r->cfInit!=NULL); return r->cfInit(i,r); }
@@ -720,7 +721,6 @@ static FORCE_INLINE void   n_CoeffWrite(const coeffs r, BOOLEAN details = TRUE)
 { assume(r != NULL); assume(r->cfCoeffWrite != NULL); r->cfCoeffWrite(r, details); }
 
 // Tests:
-#ifdef HAVE_RINGS
 static FORCE_INLINE BOOLEAN nCoeff_is_Ring_2toM(const coeffs r)
 { assume(r != NULL); return (getCoeffType(r)==n_Z2m); }
 
@@ -729,11 +729,6 @@ static FORCE_INLINE BOOLEAN nCoeff_is_Ring_PtoM(const coeffs r)
 
 static FORCE_INLINE BOOLEAN nCoeff_is_Ring(const coeffs r)
 { assume(r != NULL); return (r->is_field==0); }
-#else
-#define nCoeff_is_Ring_2toM(A) 0
-#define nCoeff_is_Ring_PtoM(A) 0
-#define nCoeff_is_Ring(A)      0
-#endif
 
 /// returns TRUE, if r is a field or r has no zero divisors (i.e is a domain)
 static FORCE_INLINE BOOLEAN nCoeff_is_Domain(const coeffs r)
@@ -752,12 +747,10 @@ static FORCE_INLINE BOOLEAN nCoeff_is_Domain(const coeffs r)
 ///                  or ((a, b <> 0) and (b/gcd(a, b) is odd))
 static FORCE_INLINE BOOLEAN n_DivBy(number a, number b, const coeffs r)
 { assume(r != NULL);
-#ifdef HAVE_RINGS
   if( nCoeff_is_Ring(r) )
   {
     assume(r->cfDivBy!=NULL); return r->cfDivBy(a,b,r);
   }
-#endif
   return !n_IsZero(b, r);
 }
 
@@ -902,7 +895,7 @@ static FORCE_INLINE BOOLEAN nCoeff_is_CF(const coeffs r)
 static FORCE_INLINE BOOLEAN nCoeff_has_simple_inverse(const coeffs r)
 { assume(r != NULL); return r->has_simple_Inverse; }
 
-/// TRUE if n_Delete/n_New are empty operations
+/// TRUE if n_Delete is empty operation
 static FORCE_INLINE BOOLEAN nCoeff_has_simple_Alloc(const coeffs r)
 { assume(r != NULL); return r->has_simple_Alloc; }
 
