@@ -199,7 +199,6 @@ static number nlMapLongR(number from, const coeffs src, const coeffs dst);
 static number nlMapR(number from, const coeffs src, const coeffs dst);
 
 
-#ifdef HAVE_RINGS
 /*2
 * convert from a GMP integer
 */
@@ -231,8 +230,6 @@ number nlMapMachineInt(number from, const coeffs /*src*/, const coeffs /*dst*/)
   z=nlShort3(z);
   return z;
 }
-#endif
-
 
 #ifdef LDEBUG
 BOOLEAN nlDBTest(number a, const char *f,const int l, const coeffs /*r*/)
@@ -1117,7 +1114,6 @@ coeffs nlQuot1(number c, const coeffs r)
   {
     rr = nInitChar(n_Zp,(void*)ch);
   }
-  #ifdef HAVE_RINGS
   else
   {
     mpz_t dummy;
@@ -1128,7 +1124,6 @@ coeffs nlQuot1(number c, const coeffs r)
     rr = nInitChar(n_Zn, (void*)&info);
     mpz_clear(dummy);
   }
-  #endif
   return(rr);
 }
 
@@ -1612,7 +1607,6 @@ number nlModP(number q, const coeffs /*Q*/, const coeffs Zp)
   return z;
 }
 
-#ifdef HAVE_RINGS
 /*2
 * convert number i (from Q) to GMP and warn if denom != 1
 */
@@ -1632,10 +1626,9 @@ void nlGMP(number &i, mpz_t n, const coeffs r)
   }
   mpz_set(n, i->z);
 }
-#endif
 
 /*2
-* acces to denominator, other 1 for integers
+* access to denominator, other 1 for integers
 */
 number nlGetDenom(number &n, const coeffs r)
 {
@@ -1664,7 +1657,7 @@ number nlGetDenom(number &n, const coeffs r)
 }
 
 /*2
-* acces to Nominator, nlCopy(n) for integers
+* access to Nominator, nlCopy(n) for integers
 */
 number nlGetNumerator(number &n, const coeffs r)
 {
@@ -1756,7 +1749,7 @@ number _nlCopy_NoImm(number a)
   {
     case 0:
     case 1:
-            mpz_init_set(b->n,a->n);
+            mpz_init_set(b->n,a->n); /*no break*/
     case 3:
             mpz_init_set(b->z,a->z);
             break;
@@ -1772,7 +1765,7 @@ void _nlDelete_NoImm(number *a)
     {
       case 0:
       case 1:
-        mpz_clear((*a)->n);
+        mpz_clear((*a)->n); /*no break*/
       case 3:
         mpz_clear((*a)->z);
     }
@@ -2508,7 +2501,6 @@ nMapFunc nlSetMap(const coeffs src, const coeffs dst)
   {
     return nlMapC; /* C -> Q */
   }
-#ifdef HAVE_RINGS
   if (src->rep==n_rep_gmp) // nCoeff_is_Z(src) || nCoeff_is_Ring_PtoM(src) || nCoeff_is_Zn(src))
   {
     return nlMapGMP;
@@ -2521,7 +2513,6 @@ nMapFunc nlSetMap(const coeffs src, const coeffs dst)
   {
     return nlMapMachineInt;
   }
-#endif
   return NULL;
 }
 /*2
@@ -2569,7 +2560,7 @@ number nlInit2gmp (mpz_t i, mpz_t j, const coeffs r)
 
 #else // DO_LINLINE
 
-// declare immedate routines
+// declare immediate routines
 number nlRInit (long i);
 BOOLEAN _nlEqual_aNoImm_OR_bNoImm(number a, number b);
 number  _nlCopy_NoImm(number a);
@@ -3093,7 +3084,7 @@ number nlExtGcd(number a, number b, number *s, number *t, const coeffs)
 
 VAR int n_SwitchChinRem=0;
 number   nlChineseRemainderSym(number *x, number *q,int rl, BOOLEAN sym, CFArray &inv_cache,const coeffs CF)
-// elemenst in the array are x[0..(rl-1)], q[0..(rl-1)]
+// elements in the array are x[0..(rl-1)], q[0..(rl-1)]
 {
   setCharacteristic( 0 ); // only in char 0
   Off(SW_RATIONAL);
