@@ -23,8 +23,6 @@
 
 #include <string.h>
 
-#ifdef HAVE_RINGS
-
 void nrnWrite (number a, const coeffs);
 #ifdef LDEBUG
 static BOOLEAN nrnDBTest      (number a, const char *f, const int l, const coeffs r);
@@ -71,7 +69,7 @@ static char* nrnCoeffName(const coeffs r)
   l+=24;
   nrnCoeffName_buff=(char*)omAlloc(l);
   s= mpz_get_str (s, 10, r->modBase);
-  int ll;
+  int ll=0;
   if (nCoeff_is_Zn(r))
   {
     if (strlen(s)<10)
@@ -1069,6 +1067,3 @@ BOOLEAN nrnInitChar (coeffs r, void* p)
   }
   return FALSE;
 }
-
-#endif
-/* #ifdef HAVE_RINGS */
