@@ -49,9 +49,8 @@ poly p_ChineseRemainder(poly *xx, number *x,number *q, int rl, CFArray &inv_cach
  *
  ***************************************************************/
 unsigned long p_GetShortExpVector(const poly a, const ring r);
-
-/// p_GetShortExpVector of p * pp
-unsigned long p_GetShortExpVector(const poly p, const poly pp, const ring r);
+unsigned long p_GetShortExpVector0(const poly a, const ring r);
+unsigned long p_GetShortExpVector1(const poly a, const ring r);
 
 #ifdef HAVE_RINGS
 /*! divisibility check over ground ring (which may contain zero divisors);
@@ -112,7 +111,7 @@ void p_ShallowDelete(poly *p, const ring r);
 
 /***************************************************************
  *
- * Copying/Deleteion of polys: args may be NULL
+ * Copying/Deletion of polys: args may be NULL
  *  - p/q as arg mean a poly
  *  - m a monomial
  *  - n a number
@@ -188,9 +187,9 @@ BOOLEAN _pp_Test(poly p, ring lmRing, ring tailRing, int level);
 /*2
 * returns the length of a polynomial (numbers of monomials)
 */
-static inline unsigned pLength(poly a)
+static inline int pLength(poly a)
 {
-  unsigned l = 0;
+  int l = 0;
   while (a!=NULL)
   {
     pIter(a);
@@ -211,7 +210,6 @@ void      p_ProjectiveUnique(poly p,const ring r);
 
 void      p_ContentForGB(poly p, const ring r);
 void      p_Content(poly p, const ring r);
-void      p_Content_n(poly p, number &c,const ring r);
 #if 1
 // currently only used by Singular/janet
 void      p_SimpleContent(poly p, int s, const ring r);
@@ -228,6 +226,8 @@ int       p_Size( poly p, const ring r );
 poly      p_Homogen (poly p, int varnum, const ring r);
 
 BOOLEAN   p_IsHomogeneous (poly p, const ring r);
+BOOLEAN   p_IsHomogeneousW (poly p, const intvec *w, const ring r);
+BOOLEAN   p_IsHomogeneousW (poly p, const intvec *w, const intvec *module_w,const ring r);
 
 // Setm
 static inline void p_Setm(poly p, const ring r)
@@ -288,7 +288,7 @@ static inline   void p_SetCompP(poly p, int i, ring lmRing, ring tailRing)
   }
 }
 
-// returns maximal column number in the modul element a (or 0)
+// returns maximal column number in the module element a (or 0)
 static inline long p_MaxComp(poly p, ring lmRing, ring tailRing)
 {
   long result,i;
@@ -373,7 +373,7 @@ void   p_String0Long(const poly p, ring lmRing, ring tailRing);
 
 /***************************************************************
  *
- * Degree stuff -- see p_polys.cc for explainations
+ * Degree stuff -- see p_polys.cc for explanations
  *
  ***************************************************************/
 
@@ -655,7 +655,7 @@ static inline int p_Comp_k_n(poly a, poly b, int k, ring r)
 
 /***************************************************************
  *
- * Allocation/Initalization/Deletion
+ * Allocation/Initialization/Deletion
  *
  ***************************************************************/
 #if (OM_TRACK > 2) && defined(OM_TRACK_CUSTOM)
@@ -677,16 +677,20 @@ static inline poly p_New(ring r)
   return p_New(r, r->PolyBin);
 }
 
-#if PDEBUG > 2
+#if (PDEBUG > 2) || defined(XALLOC_BIN)
 static inline void p_LmFree(poly p, ring r)
 #else
 static inline void p_LmFree(poly p, ring)
 #endif
 {
   p_LmCheckPolyRing2(p, r);
+  #ifdef XALLOC_BIN
+  omFreeBin(p,r->PolyBin);
+  #else
   omFreeBinAddr(p);
+  #endif
 }
-#if PDEBUG > 2
+#if (PDEBUG > 2) || defined(XALLOC_BIN)
 static inline void p_LmFree(poly *p, ring r)
 #else
 static inline void p_LmFree(poly *p, ring)
@@ -695,9 +699,13 @@ static inline void p_LmFree(poly *p, ring)
   p_LmCheckPolyRing2(*p, r);
   poly h = *p;
   *p = pNext(h);
+  #ifdef XALLOC_BIN
+  omFreeBin(h,r->PolyBin);
+  #else
   omFreeBinAddr(h);
+  #endif
 }
-#if PDEBUG > 2
+#if (PDEBUG > 2) || defined(XALLOC_BIN)
 static inline poly p_LmFreeAndNext(poly p, ring r)
 #else
 static inline poly p_LmFreeAndNext(poly p, ring)
@@ -705,20 +713,32 @@ static inline poly p_LmFreeAndNext(poly p, ring)
 {
   p_LmCheckPolyRing2(p, r);
   poly pnext = pNext(p);
+  #ifdef XALLOC_BIN
+  omFreeBin(p,r->PolyBin);
+  #else
   omFreeBinAddr(p);
+  #endif
   return pnext;
 }
 static inline void p_LmDelete(poly p, const ring r)
 {
   p_LmCheckPolyRing2(p, r);
   n_Delete(&pGetCoeff(p), r->cf);
+  #ifdef XALLOC_BIN
+  omFreeBin(p,r->PolyBin);
+  #else
   omFreeBinAddr(p);
+  #endif
 }
 static inline void p_LmDelete0(poly p, const ring r)
 {
   p_LmCheckPolyRing2(p, r);
   if (pGetCoeff(p)!=NULL) n_Delete(&pGetCoeff(p), r->cf);
+  #ifdef XALLOC_BIN
+  omFreeBin(p,r->PolyBin);
+  #else
   omFreeBinAddr(p);
+  #endif
 }
 static inline void p_LmDelete(poly *p, const ring r)
 {
@@ -726,14 +746,22 @@ static inline void p_LmDelete(poly *p, const ring r)
   poly h = *p;
   *p = pNext(h);
   n_Delete(&pGetCoeff(h), r->cf);
+  #ifdef XALLOC_BIN
+  omFreeBin(h,r->PolyBin);
+  #else
   omFreeBinAddr(h);
+  #endif
 }
 static inline poly p_LmDeleteAndNext(poly p, const ring r)
 {
   p_LmCheckPolyRing2(p, r);
   poly pnext = pNext(p);
   n_Delete(&pGetCoeff(p), r->cf);
+  #ifdef XALLOC_BIN
+  omFreeBin(p,r->PolyBin);
+  #else
   omFreeBinAddr(p);
+  #endif
   return pnext;
 }
 
@@ -895,7 +923,7 @@ static inline void p_Delete(poly *p,  const ring lmRing, const ring tailRing)
   }
 }
 
-// copys monomials of p, allocates new monomials from bin,
+// copies monomials of p, allocates new monomials from bin,
 // deletes monomials of p
 static inline poly p_ShallowCopyDelete(poly p, const ring r, omBin bin)
 {
@@ -1058,14 +1086,14 @@ static inline poly p_Minus_mm_Mult_qq(poly p, const poly m, const poly q, const
 }
 
 
-// returns p*Coeff(m) for such monomials pm of p, for which m is divisble by pm
+// returns p*Coeff(m) for such monomials pm of p, for which m is divisible by pm
 static inline poly pp_Mult_Coeff_mm_DivSelect(poly p, const poly m, const ring r)
 {
   int shorter;
   return r->p_Procs->pp_Mult_Coeff_mm_DivSelect(p, m, shorter, r);
 }
 
-// returns p*Coeff(m) for such monomials pm of p, for which m is divisble by pm
+// returns p*Coeff(m) for such monomials pm of p, for which m is divisible by pm
 // if lp is length of p on input then lp is length of returned poly on output
 static inline poly pp_Mult_Coeff_mm_DivSelect(poly p, int &lp, const poly m, const ring r)
 {
@@ -1081,18 +1109,22 @@ static inline poly p_Neg(poly p, const ring r)
   return r->p_Procs->p_Neg(p, r);
 }
 
-extern poly  _p_Mult_q(poly p, poly q, const int copy, const ring r);
+poly _p_Mult_q(poly p, poly q, const int copy, const ring r);
+#ifdef HAVE_RINGS
+poly _p_Mult_q_Normal_ZeroDiv(poly p, poly q, const int copy, const ring r);
+#endif
+
 // returns p*q, destroys p and q
 static inline poly p_Mult_q(poly p, poly q, const ring r)
 {
   assume( (p != q) || (p == NULL && q == NULL) );
 
-  if (p == NULL)
+  if (UNLIKELY(p == NULL))
   {
     p_Delete(&q, r);
     return NULL;
   }
-  if (q == NULL)
+  if (UNLIKELY(q == NULL))
   {
     p_Delete(&p, r);
     return NULL;
@@ -1112,9 +1144,14 @@ static inline poly p_Mult_q(poly p, poly q, const ring r)
     return p;
   }
 #if defined(HAVE_PLURAL) || defined(HAVE_SHIFTBBA)
-  if (rIsNCRing(r))
+  if (UNLIKELY(rIsNCRing(r)))
     return _nc_p_Mult_q(p, q, r);
   else
+#endif
+#ifdef HAVE_RINGS
+  if (UNLIKELY(!nCoeff_is_Domain(r->cf)))
+    return _p_Mult_q_Normal_ZeroDiv(p, q, 0, r);
+  else
 #endif
   return _p_Mult_q(p, q, 0, r);
 }
@@ -1122,7 +1159,7 @@ static inline poly p_Mult_q(poly p, poly q, const ring r)
 // returns p*q, does neither destroy p nor q
 static inline poly pp_Mult_qq(poly p, poly q, const ring r)
 {
-  if (p == NULL || q == NULL) return NULL;
+  if (UNLIKELY(p == NULL || q == NULL)) return NULL;
 
   if (pNext(p) == NULL)
   {
@@ -1135,18 +1172,23 @@ static inline poly pp_Mult_qq(poly p, poly q, const ring r)
   }
 
   poly qq = q;
-  if (p == q)
+  if (UNLIKELY(p == q))
     qq = p_Copy(q, r);
 
   poly res;
 #if defined(HAVE_PLURAL) || defined(HAVE_SHIFTBBA)
-  if (rIsNCRing(r))
+  if (UNLIKELY(rIsNCRing(r)))
     res = _nc_pp_Mult_qq(p, qq, r);
   else
+#endif
+#ifdef HAVE_RINGS
+  if (UNLIKELY(!nCoeff_is_Domain(r->cf)))
+    res = _p_Mult_q_Normal_ZeroDiv(p, qq, 1, r);
+  else
 #endif
     res = _p_Mult_q(p, qq, 1, r);
 
-  if (qq != q)
+  if (UNLIKELY(qq != q))
     p_Delete(&qq, r);
   return res;
 }
@@ -1187,7 +1229,7 @@ static inline poly p_Merge_q(poly p, poly q, const ring r)
   return r->p_Procs->p_Merge_q(p, q, r);
 }
 
-// like p_SortMerge, except that p may have equal monimals
+// like p_SortMerge, except that p may have equal monomials
 static inline poly p_SortAdd(poly p, const ring r, BOOLEAN revert= FALSE)
 {
   if (revert) p = pReverse(p);
@@ -1257,7 +1299,7 @@ while(0)
 
 /***************************************************************
  *
- * Allocation/Initalization/Deletion
+ * Allocation/Initialization/Deletion
  *
  ***************************************************************/
 // adjustments for negative weights
@@ -1637,23 +1679,16 @@ static inline int p_LtCmpNoAbs(poly p, poly q, const ring r)
 
 #ifdef HAVE_RINGS
 // This is the equivalent of pLmCmp(p,q) != -currRing->OrdSgn for rings
-// It is used in posInLRing and posInTRing
+// It is used in posInTRing
 static inline int p_LtCmpOrdSgnDiffM(poly p, poly q, const ring r)
 {
-  if(r->OrdSgn == 1)
-  {
-    return(p_LtCmp(p,q,r) == 1);
-  }
-  else
-  {
-    return(p_LmCmp(p,q,r) == -1);
-  }
+  return(p_LtCmp(p,q,r) == r->OrdSgn);
 }
 #endif
 
 #ifdef HAVE_RINGS
 // This is the equivalent of pLmCmp(p,q) != currRing->OrdSgn for rings
-// It is used in posInLRing and posInTRing
+// It is used in posInTRing
 static inline int p_LtCmpOrdSgnDiffP(poly p, poly q, const ring r)
 {
   if(r->OrdSgn == 1)
@@ -1664,13 +1699,12 @@ static inline int p_LtCmpOrdSgnDiffP(poly p, poly q, const ring r)
   {
     return(p_LtCmp(p,q,r) != -1);
   }
-
 }
 #endif
 
 #ifdef HAVE_RINGS
 // This is the equivalent of pLmCmp(p,q) == -currRing->OrdSgn for rings
-// It is used in posInLRing and posInTRing
+// It is used in posInTRing
 static inline int p_LtCmpOrdSgnEqM(poly p, poly q, const ring r)
 {
   return(p_LtCmp(p,q,r) == -r->OrdSgn);
@@ -1679,7 +1713,7 @@ static inline int p_LtCmpOrdSgnEqM(poly p, poly q, const ring r)
 
 #ifdef HAVE_RINGS
 // This is the equivalent of pLmCmp(p,q) == currRing->OrdSgn for rings
-// It is used in posInLRing and posInTRing
+// It is used in posInTRing
 static inline int p_LtCmpOrdSgnEqP(poly p, poly q, const ring r)
 {
   return(p_LtCmp(p,q,r) == r->OrdSgn);
@@ -1799,7 +1833,9 @@ static inline BOOLEAN _p_LmDivisibleByNoComp(poly a, const ring r_a, poly b, con
   do
   {
     if (p_GetExp(a,i,r_a) > p_GetExp(b,i,r_b))
+    {
       return FALSE;
+    }
     i--;
   }
   while (i);
@@ -1852,12 +1888,6 @@ static inline BOOLEAN _p_LmDivisibleBy(poly a, poly b, const ring r)
     return _p_LmDivisibleByNoComp(a, b, r);
   return FALSE;
 }
-static inline BOOLEAN _p_LmDivisibleBy(poly a, const ring r_a, poly b, const ring r_b)
-{
-  if (p_GetComp(a, r_a) == 0 || p_GetComp(a,r_a) == p_GetComp(b,r_b))
-    return _p_LmDivisibleByNoComp(a, r_a, b, r_b);
-  return FALSE;
-}
 static inline BOOLEAN p_LmDivisibleByNoComp(poly a, poly b, const ring r)
 {
   p_LmCheckPolyRing1(a, r);
@@ -1890,21 +1920,6 @@ static inline BOOLEAN p_DivisibleBy(poly a, poly b, const ring r)
       return _p_LmDivisibleByNoComp(a,b,r);
   return FALSE;
 }
-static inline BOOLEAN p_DivisibleBy(poly a, const ring r_a, poly b, const ring r_b)
-{
-  pIfThen1(b!=NULL, p_LmCheckPolyRing1(b, r_b));
-  pIfThen1(a!=NULL, p_LmCheckPolyRing1(a, r_a));
-  if (a != NULL) {
-      return _p_LmDivisibleBy(a, r_a, b, r_b);
-  }
-  return FALSE;
-}
-static inline BOOLEAN p_LmDivisibleBy(poly a, const ring r_a, poly b, const ring r_b)
-{
-  p_LmCheckPolyRing(a, r_a);
-  p_LmCheckPolyRing(b, r_b);
-  return _p_LmDivisibleBy(a, r_a, b, r_b);
-}
 
 static inline BOOLEAN p_LmShortDivisibleBy(poly a, unsigned long sev_a,
                                     poly b, unsigned long not_sev_b, const ring r)
@@ -1946,26 +1961,6 @@ static inline BOOLEAN p_LmShortDivisibleByNoComp(poly a, unsigned long sev_a,
 #endif
 }
 
-static inline BOOLEAN p_LmShortDivisibleBy(poly a, unsigned long sev_a, const ring r_a,
-                                      poly b, unsigned long not_sev_b, const ring r_b)
-{
-  p_LmCheckPolyRing1(a, r_a);
-  p_LmCheckPolyRing1(b, r_b);
-#ifndef PDIV_DEBUG
-  _pPolyAssume2(p_GetShortExpVector(a, r_a) == sev_a, a, r_a);
-  _pPolyAssume2(p_GetShortExpVector(b, r_b) == ~ not_sev_b, b, r_b);
-
-  if (sev_a & not_sev_b)
-  {
-    pAssume1(_p_LmDivisibleByNoComp(a, r_a, b, r_b) == FALSE);
-    return FALSE;
-  }
-  return _p_LmDivisibleBy(a, r_a, b, r_b);
-#else
-  return pDebugLmShortDivisibleBy(a, sev_a, r_a, b, not_sev_b, r_b);
-#endif
-}
-
 /***************************************************************
  *
  * Misc things on Lm
@@ -2079,10 +2074,9 @@ poly      p_PolyDiv(poly &p, const poly divisor, const BOOLEAN needResult, const
 /* syszygy stuff */
 BOOLEAN   p_VectorHasUnitB(poly p, int * k, const ring r);
 void      p_VectorHasUnit(poly p, int * k, int * len, const ring r);
-poly      p_TakeOutComp1(poly * p, int k, const ring r);
-// Splits *p into two polys: *q which consists of all monoms with
-// component == comp and *p of all other monoms *lq == pLength(*q)
-// On return all components pf *q == 0
+/// Splits *p into two polys: *q which consists of all monoms with
+/// component == comp and *p of all other monoms *lq == pLength(*q)
+/// On return all components pf *q == 0
 void p_TakeOutComp(poly *p, long comp, poly *q, int *lq, const ring r);
 
 // This is something weird -- Don't use it, unless you know what you are doing
@@ -2107,6 +2101,7 @@ void p_SetModDeg(intvec *w, ring r);
 
 /*------------ Jet ----------------------------------*/
 poly pp_Jet(poly p, int m, const ring R);
+poly pp_Jet0(poly p, const ring R); /*pp_Jet(p,0,R)*/
 poly p_Jet(poly p, int m,const ring R);
 poly pp_JetW(poly p, int m, int *w, const ring R);
 poly p_JetW(poly p, int m, int *w, const ring R);
