@@ -27,7 +27,7 @@
 VAR omBin sip_sideal_bin = omGetSpecBin(sizeof(sip_sideal));
 
 STATIC_VAR poly * idpower;
-/*collects the monomials in makemonoms, must be allocated befor*/
+/*collects the monomials in makemonoms, must be allocated before*/
 STATIC_VAR int idpowerpoint;
 /*index of the actual monomial in idpower*/
 
@@ -53,7 +53,7 @@ ideal idInit(int idsize, int rank)
 
 #ifdef PDEBUG
 // this is only for outputting an ideal within the debugger
-// therefor it accept the otherwise illegal id==NULL
+// therefore it accept the otherwise illegal id==NULL
 void idShow(const ideal id, const ring lmRing, const ring tailRing, const int debugPrint)
 {
   assume( debugPrint >= 0 );
@@ -154,22 +154,22 @@ void id_Delete (ideal * h, ring r)
 
 void id_Delete0 (ideal * h, ring r)
 {
-  const long elems = IDELEMS(*h);
+  long j = IDELEMS(*h);
 
- assume( (*h)->m != NULL );
+  if(j>0)
+  {
+    do
+    {
+      j--;
+      poly pp=((*h)->m[j]);
+      if (pp!=NULL) p_Delete(&pp, r);
+    }
+    while (j>0);
+    omFree((ADDRESS)((*h)->m));
+  }
 
- long j = elems;
- do
- {
-   j--;
-   poly pp=((*h)->m[j]);
-   if (pp!=NULL) p_Delete(&pp, r);
- }
- while (j>0);
-
- omFree((ADDRESS)((*h)->m));
- omFreeBin((ADDRESS)*h, sip_sideal_bin);
- *h=NULL;
+  omFreeBin((ADDRESS)*h, sip_sideal_bin);
+  *h=NULL;
 }
 
 
@@ -215,6 +215,7 @@ void idSkipZeroes (ideal ide)
       if (change)
       {
         ide->m[j] = ide->m[k];
+        ide->m[k] = NULL;
       }
     }
     else
@@ -226,18 +227,13 @@ void idSkipZeroes (ideal ide)
   {
     if (j == -1)
       j = 0;
-    else
-    {
-      for (k=j+1; k<idelems; k++)
-        ide->m[k] = NULL;
-    }
     j++;
     pEnlargeSet(&(ide->m),idelems,j-idelems);
     IDELEMS(ide) = j;
   }
 }
 
-int idSkipZeroes0 (ideal ide)
+int idSkipZeroes0 (ideal ide) /*idSkipZeroes without realloc*/
 {
   assume (ide != NULL);
 
@@ -963,14 +959,34 @@ ideal  id_Mult (ideal h1,ideal  h2, const ring R)
 BOOLEAN idIs0 (ideal h)
 {
   assume (h != NULL); // will fail :(
-//  if (h == NULL) return TRUE;
-
-  for( int i = IDELEMS(h)-1; i >= 0; i-- )
-    if(h->m[i] != NULL)
-      return FALSE;
 
+  if (h->m!=NULL)
+  {
+    for( int i = IDELEMS(h)-1; i >= 0; i-- )
+      if(h->m[i] != NULL)
+        return FALSE;
+  }
   return TRUE;
+}
 
+/// returns true if h is generated by monomials
+BOOLEAN idIsMonomial (ideal h)
+{
+  assume (h != NULL);
+
+  BOOLEAN found_mon=FALSE;
+  if (h->m!=NULL)
+  {
+    for( int i = IDELEMS(h)-1; i >= 0; i-- )
+    {
+      if(h->m[i] != NULL)
+      {
+        if(pNext(h->m[i])!=NULL) return FALSE;
+        found_mon=TRUE;
+      }
+    }
+  }
+  return found_mon;
 }
 
 /// return the maximal component number found in any polynomial in s
@@ -993,8 +1009,27 @@ long id_RankFreeModule (ideal s, ring lmRing, ring tailRing)
   return j; //  return -1;
 }
 
+BOOLEAN id_IsModule(ideal A, const ring src)
+{
+  if ((src->VarOffset[0]== -1)
+  || (src->pCompIndex<0))
+    return FALSE; // ring without components
+  for (int i=IDELEMS(A)-1;i>=0;i--)
+  {
+    if (A->m[i]!=NULL)
+    {
+      if (p_GetComp(A->m[i],src)>0)
+        return TRUE;
+      else
+        return FALSE;
+    }
+  }
+  return A->rank>1;
+}
+
+
 /*2
-*returns true if id is homogenous with respect to the aktual weights
+*returns true if id is homogeneous with respect to the actual weights
 */
 BOOLEAN id_HomIdeal (ideal id, ideal Q, const ring r)
 {
@@ -1019,6 +1054,32 @@ BOOLEAN id_HomIdeal (ideal id, ideal Q, const ring r)
   return b;
 }
 
+/*2
+*returns true if id is homogeneous with respect to totaldegree
+*/
+BOOLEAN id_HomIdealDP (ideal id, ideal Q, const ring r)
+{
+  int i;
+  BOOLEAN b;
+  i = 0;
+  b = TRUE;
+  while ((i < IDELEMS(id)) && b)
+  {
+    b = p_IsHomogeneousDP(id->m[i],r);
+    i++;
+  }
+  if ((b) && (Q!=NULL) && (IDELEMS(Q)>0))
+  {
+    i=0;
+    while ((i < IDELEMS(Q)) && b)
+    {
+      b = p_IsHomogeneousDP(Q->m[i],r);
+      i++;
+    }
+  }
+  return b;
+}
+
 BOOLEAN id_HomIdealW (ideal id, ideal Q,  const intvec *w, const ring r)
 {
   int i;
@@ -1452,6 +1513,18 @@ ideal id_Homogen(ideal h, int varnum,const ring r)
   return m;
 }
 
+ideal id_HomogenDP(ideal h, int varnum,const ring r)
+{
+  ideal m = idInit(IDELEMS(h),h->rank);
+  int i;
+
+  for (i=IDELEMS(h)-1;i>=0; i--)
+  {
+    m->m[i]=p_HomogenDP(h->m[i],varnum,r);
+  }
+  return m;
+}
+
 /*------------------type conversions----------------*/
 ideal id_Vec2Ideal(poly vec, const ring R)
 {
@@ -1517,7 +1590,7 @@ ideal id_Matrix2Module(matrix mat, const ring R)
 }
 
 /*2
-* converts a module into a matrix, destroyes the input
+* converts a module into a matrix, destroys the input
 */
 matrix id_Module2Matrix(ideal mod, const ring R)
 {
@@ -1560,7 +1633,7 @@ matrix id_Module2Matrix(ideal mod, const ring R)
       MATELEM0(result,cp-1,i) = p_Add_q(MATELEM0(result,cp-1,i),h,R);
     }
   }
-  // obachman 10/99: added the following line, otherwise memory leack!
+  // obachman 10/99: added the following line, otherwise memory leak!
   id_Delete(&mod,R);
   return result;
 }
@@ -1785,6 +1858,19 @@ ideal id_Jet(const ideal i,int d, const ring R)
   return r;
 }
 
+ideal id_Jet0(const ideal i, const ring R)
+{
+  ideal r=idInit((i->nrows)*(i->ncols),i->rank);
+  r->nrows = i-> nrows;
+  r->ncols = i-> ncols;
+  //r->rank = i-> rank;
+
+  for(long k=((long)(i->nrows))*((long)(i->ncols))-1;k>=0; k--)
+    r->m[k]=pp_Jet0(i->m[k],R);
+
+  return r;
+}
+
 ideal id_JetW(const ideal i,int d, intvec * iv, const ring R)
 {
   ideal r=idInit(IDELEMS(i),i->rank);
@@ -1805,63 +1891,6 @@ ideal id_JetW(const ideal i,int d, intvec * iv, const ring R)
   return r;
 }
 
-/*3
-* searches for the next unit in the components of the module arg and
-* returns the first one;
-*/
-int id_ReadOutPivot(ideal arg,int* comp, const ring r)
-{
-  if (idIs0(arg)) return -1;
-  int i=0,j, generator=-1;
-  int rk_arg=arg->rank; //idRankFreeModule(arg);
-  int * componentIsUsed =(int *)omAlloc((rk_arg+1)*sizeof(int));
-  poly p;
-
-  while ((generator<0) && (i<IDELEMS(arg)))
-  {
-    memset(componentIsUsed,0,(rk_arg+1)*sizeof(int));
-    p = arg->m[i];
-    while (p!=NULL)
-    {
-      j = __p_GetComp(p,r);
-      if (componentIsUsed[j]==0)
-      {
-        if (p_LmIsConstantComp(p,r) &&
-            (!rField_is_Ring(r) || n_IsUnit(pGetCoeff(p),r->cf)))
-        {
-          generator = i;
-          componentIsUsed[j] = 1;
-        }
-        else
-        {
-          componentIsUsed[j] = -1;
-        }
-      }
-      else if (componentIsUsed[j]>0)
-      {
-        (componentIsUsed[j])++;
-      }
-      pIter(p);
-    }
-    i++;
-  }
-  i = 0;
-  *comp = -1;
-  for (j=0;j<=rk_arg;j++)
-  {
-    if (componentIsUsed[j]>0)
-    {
-      if ((*comp==-1) || (componentIsUsed[j]<i))
-      {
-        *comp = j;
-        i= componentIsUsed[j];
-      }
-    }
-  }
-  omFree(componentIsUsed);
-  return generator;
-}
-
 #if 0
 static void idDeleteComp(ideal arg,int red_comp)
 {
@@ -2096,7 +2125,7 @@ ideal id_TensorModuleMult(const int m, const ideal M, const ring rRing)
       p_IncrExp(h, vv, rRing); // h *= var(j) && //      p_AddExp(h, vv, 1, rRing);
       p_SetComp(h, cc, rRing);
 
-      p_Setm(h, rRing);         // addjust degree after the previous steps!
+      p_Setm(h, rRing);         // adjust degree after the previous steps!
 
       pTempSum = p_Add_q(pTempSum, h, rRing); // it is slow since h will be usually put to the back of pTempSum!!!
 
