@@ -15,9 +15,15 @@
 #define OM_MALLOC_VFREE(addr, size) OM_MALLOC_FREE(addr)
 #ifdef HAVE_MALLOC_SIZE
 /* the following will work under Mac OS X */
-#define OM_MALLOC_SIZEOF_ADDR(addr)  (malloc_size(addr))
+  #ifdef HAVE_MALLOC_H
+  #include <malloc.h>
+  #elif defined(HAVE_MALLOC_MALLOC_H)
+  #include <malloc/malloc.h>
+  #endif
+  #define OM_MALLOC_SIZEOF_ADDR(addr)  (malloc_size(addr))
 #elif (defined(HAVE_MALLOC_USABLE_SIZE))
 /* and this will work under Linux */
+#include <stdlib.h>
 #define OM_MALLOC_SIZEOF_ADDR(addr) (malloc_usable_size(addr))
 #else
 static inline size_t OM_MALLOC_SIZEOF_ADDR(void *d) { long*dd=(long*)d; return *(dd-1); }
